{-# OPTIONS -fglasgow-exts -#include Button.h -#include CheckBox.h -#include EditBox.h -#include Label.h -#include ListBox.h -#include PopUp.h -#include RadioBox.h -#include Window.h -#include ProgressBar.h -#include Slider.h -#include Notebook.h -#include GroupBox.h #-}
-----------------------------------------------------------------------------------------
{-| Module      :  Controls
    Copyright   :  (c) Krasimir Angelov 2003
    License     :  BSD-style

    Maintainer  :  ka2_mail@yahoo.com
    Stability   :  provisional
    Portability :  portable

    Defines common controls:

    * Labels

    * Button
    
    * Check box
    
    * Edit box
    
    * List box
    
    * Popup box
    
    * Radio item
    
    * Slider
    
    * ProgressBar
    
    * CompoundControl
    
    * GroupBox
    
    * Notebook
    
    * NotebookPage
-}
-----------------------------------------------------------------------------------------
module Graphics.UI.Port.Controls
           ( 
           -- * Generic
             moveResizeControl
           , getControlFrame
           , setControlEnabled, getControlEnabled
           , setControlVisible, getControlVisible
           , setControlTip,     getControlTip
           , relayoutContainer
           -- * Label
           , createLabel, getLabelRequestSize
           , getLabelText, setLabelText
           , changeLabelFont
           -- * Button
           , createButton, getButtonRequestSize
           , getButtonText, setButtonText
           , changeButtonFont
           -- * Edit box
           , createEdit, getEditRequestSize
           , getEditText,      setEditText
           , getEditReadOnly,  setEditReadOnly
           , getEditPassword,setEditPassword
           , changeEditBoxFont
           -- * Check box
           , createCheckBox, getCheckBoxRequestSize
           , getCheckBoxText,  setCheckBoxText
           , getCheckBoxSelectState
           , setCheckBoxSelectState
           -- * Radio item
           , createRadioBox, getRadioBoxRequestSize
           , getRadioBoxText,  setRadioBoxText
           , setRadioBoxSelectState
           , getRadioBoxSelectState
           , setRadioBoxGroup
           -- * Pop up box
           , createPopUp, getPopUpRequestSize
           , appendPopUpItem, insertPopUpItem
           , removePopUpItem, removeAllPopUpItems
           , getPopUpSelection, setPopUpSelection
           -- * List box
           , createListBox, getListBoxRequestSize
           , appendListBoxItem, insertListBoxItem
           , removeListBoxItem, removeAllListBoxItems
           , getListBoxSingleSelection, setListBoxSingleSelection
           , getListBoxItemSelectState, setListBoxItemSelectState
	   , getListBoxCurrentItem
           -- * Slider
           , createHorzSlider, createVertSlider, getSliderRequestSize
           , getSliderRange,    setSliderRange
           , getSliderPosition, setSliderPosition
           -- * ProgressBar
	   , createHorzProgressBar, createVertProgressBar, getProgressBarRequestSize
	   , setProgressBarFraction, getProgressBarFraction
	   -- * CompoundControl
	   , createCompoundControl, getCompoundControlRequestSize
	   -- * GroupBox
	   , createGroupBox, getGroupBoxBordersSize
	   , getGroupBoxText, setGroupBoxText
	   -- * Notebook
	   , createNotebook, getNotebookRequestSize
	   , getNotebookLabelsPosition, setNotebookLabelsPosition
	   , getNotebookSelection,      setNotebookSelection
	   , getNotebookPageCount
	   -- * NotebookPage
	   , insertNotebookPage
	   , getNotebookPageTitle,  setNotebookPageTitle
	   , getNotebookPageBitmap, setNotebookPageBitmap
	   , getNotebookPagePos
	   , destroyNotebookPage
	   , getNotebookPageSize
           ) where

import Foreign
import Foreign.C
import Control.Concurrent.MVar
import Data.Maybe(fromMaybe)
import Graphics.UI.Port.Types
import Graphics.UI.Port.Handlers  -- just for haddock
import Graphics.UI.Port.PtrMap as PtrMap

-----------------------------------------------------------------------------------------
-- Generic  
-----------------------------------------------------------------------------------------}
-- | Change the position and size of a control.
moveResizeControl :: WindowHandle -> Rect -> IO ()
moveResizeControl hwnd rect
  = withCRect rect $ \x0 y0 x1 y1 ->
    osMoveResizeControl hwnd x0 y0 (x1-x0) (y1-y0)
foreign import ccall osMoveResizeControl :: WindowHandle -> CInt -> CInt -> CInt -> CInt -> IO ()

-- | Get the position and size of a control.
getControlFrame :: WindowHandle -> IO Rect
getControlFrame hwnd
  = withCRectResult $ \crect ->
    do osGetControlRect hwnd crect
foreign import ccall osGetControlRect  :: WindowHandle -> Ptr CInt -> IO ()

foreign import ccall unsafe "osSetControlEnabled" setControlEnabled :: WindowHandle -> Bool -> IO ()
foreign import ccall unsafe "osGetControlEnabled" getControlEnabled :: WindowHandle -> IO Bool

foreign import ccall unsafe "osSetControlVisible" setControlVisible :: WindowHandle -> Bool -> IO ()
foreign import ccall unsafe "osGetControlVisible" getControlVisible :: WindowHandle -> IO Bool

getControlTip :: WindowHandle -> IO String
getControlTip hwnd
  = resultCString (osGetControlTip hwnd)
foreign import ccall osGetControlTip :: WindowHandle -> IO CString

setControlTip :: WindowHandle -> String -> IO ()
setControlTip hwnd txt
  = withCString txt (osSetControlTip hwnd)
foreign import ccall osSetControlTip :: WindowHandle -> CString -> IO ()

foreign import ccall "osReLayoutContainer" relayoutContainer :: WindowHandle -> IO ()

-----------------------------------------------------------------------------------------
-- Label  
-----------------------------------------------------------------------------------------
-- | Create a label (i.e. just text).
foreign import ccall "osCreateLabel" createLabel :: WindowHandle -> IO WindowHandle

-- | The minimal size that the label needs.
getLabelRequestSize :: WindowHandle -> IO Size
getLabelRequestSize hwnd = withCSizeResult (osGetLabelReqSize hwnd)
foreign import ccall osGetLabelReqSize :: WindowHandle -> Ptr CInt -> IO ()

getLabelText :: WindowHandle -> IO String
getLabelText hwnd = resultCString (osGetLabelText hwnd)
foreign import ccall osGetLabelText :: WindowHandle -> IO CString

setLabelText :: WindowHandle -> String -> IO ()
setLabelText hwnd txt = withCString txt (osSetLabelText hwnd)
foreign import ccall osSetLabelText :: WindowHandle -> CString -> IO ()

changeLabelFont :: WindowHandle -> Font -> IO ()
changeLabelFont hwnd font = withCFont font (osChangeLabelFont hwnd)
foreign import ccall osChangeLabelFont :: WindowHandle -> FontHandle -> IO ()

-----------------------------------------------------------------------------------------
-- Button  
-----------------------------------------------------------------------------------------
-- | Create a button control. An event handler for button clicks can be
-- installed with 'setControlCommandHandler'.
foreign import ccall "osCreateButton" createButton :: WindowHandle -> IO WindowHandle

-- | The minimal size that the button needs.
getButtonRequestSize :: WindowHandle -> IO Size
getButtonRequestSize hwnd = withCSizeResult (osGetButtonReqSize hwnd)
foreign import ccall osGetButtonReqSize :: WindowHandle -> Ptr CInt -> IO ()

getButtonText :: WindowHandle -> IO String
getButtonText hwnd = resultCString (osGetButtonText hwnd)
foreign import ccall osGetButtonText :: WindowHandle -> IO CString

setButtonText :: WindowHandle -> String -> IO ()
setButtonText hwnd txt = withCString txt (osSetButtonText hwnd)
foreign import ccall osSetButtonText :: WindowHandle -> CString -> IO ()

changeButtonFont :: WindowHandle -> Font -> IO ()
changeButtonFont hwnd font = withCFont font (osChangeButtonFont hwnd)
foreign import ccall osChangeButtonFont :: WindowHandle -> FontHandle -> IO ()

-----------------------------------------------------------------------------------------
-- CheckBox  
-----------------------------------------------------------------------------------------
-- | Create a check box. An event handler for check box clicks can be
-- installed with 'setControlCommandHandler'.
foreign import ccall "osCreateCheckBox" createCheckBox :: WindowHandle -> IO WindowHandle

getCheckBoxRequestSize :: WindowHandle -> IO Size
getCheckBoxRequestSize hwnd = withCSizeResult (osGetCheckBoxReqSize hwnd)
foreign import ccall osGetCheckBoxReqSize :: WindowHandle -> Ptr CInt -> IO ()

getCheckBoxText :: WindowHandle -> IO String
getCheckBoxText hwnd = resultCString (osGetCheckBoxText hwnd)
foreign import ccall osGetCheckBoxText :: WindowHandle -> IO CString

setCheckBoxText :: WindowHandle -> String -> IO ()
setCheckBoxText hwnd txt = withCString txt (osSetCheckBoxText hwnd)
foreign import ccall osSetCheckBoxText :: WindowHandle -> CString -> IO ()

getCheckBoxSelectState :: WindowHandle -> IO Bool
getCheckBoxSelectState hwnd
  = do x <- osGetCheckBoxState hwnd; return (fromCBool x)
foreign import ccall osGetCheckBoxState :: WindowHandle -> IO CBool

setCheckBoxSelectState :: WindowHandle -> Bool -> IO ()
setCheckBoxSelectState hwnd x
  = osSetCheckBoxState hwnd (toCBool x)
foreign import ccall osSetCheckBoxState :: WindowHandle -> CBool -> IO ()


-----------------------------------------------------------------------------------------
-- EditBox  
-----------------------------------------------------------------------------------------
-- | Create an edit control.
foreign import ccall "osCreateEdit" createEdit :: WindowHandle -> IO WindowHandle

getEditRequestSize :: WindowHandle -> IO Size
getEditRequestSize hwnd = withCSizeResult (osGetEditReqSize hwnd)
foreign import ccall osGetEditReqSize :: WindowHandle -> Ptr CInt -> IO ()

getEditText :: WindowHandle -> IO String
getEditText hwnd
  = resultCString (osGetEditText hwnd)
foreign import ccall osGetEditText :: WindowHandle -> IO CString

setEditText :: WindowHandle -> String -> IO ()
setEditText hwnd txt
  = withCString txt $ \ctxt -> osSetEditText hwnd ctxt
foreign import ccall osSetEditText :: WindowHandle -> CString -> IO ()

foreign import ccall "osSetEditReadOnly" setEditReadOnly :: WindowHandle -> Bool -> IO ()
foreign import ccall "osGetEditReadOnly" getEditReadOnly :: WindowHandle -> IO Bool

foreign import ccall "osSetEditPassword" setEditPassword :: WindowHandle -> Bool -> IO ()
foreign import ccall "osGetEditPassword" getEditPassword :: WindowHandle -> IO Bool

changeEditBoxFont :: WindowHandle -> Font -> IO ()
changeEditBoxFont hwnd font = withCFont font (osChangeEditBoxFont hwnd)
foreign import ccall osChangeEditBoxFont :: WindowHandle -> FontHandle -> IO ()

-----------------------------------------------------------------------------------------
--  RadioBox
-----------------------------------------------------------------------------------------
-- | Create a radio box.
-- An event handler for radio box clicks can be
-- installed with 'setControlCommandHandler'. 
foreign import ccall "osCreateRadioBox" createRadioBox :: WindowHandle -> IO WindowHandle

getRadioBoxRequestSize :: WindowHandle -> IO Size
getRadioBoxRequestSize hwnd = withCSizeResult (osGetRadioBoxReqSize hwnd)
foreign import ccall osGetRadioBoxReqSize :: WindowHandle -> Ptr CInt -> IO ()

getRadioBoxText :: WindowHandle -> IO String
getRadioBoxText hwnd = resultCString (osGetRadioBoxText hwnd)
foreign import ccall osGetRadioBoxText :: WindowHandle -> IO CString

setRadioBoxText :: WindowHandle -> String -> IO ()
setRadioBoxText hwnd txt = withCString txt (osSetRadioBoxText hwnd)
foreign import ccall osSetRadioBoxText :: WindowHandle -> CString -> IO ()

getRadioBoxSelectState :: WindowHandle -> IO Bool
getRadioBoxSelectState hwnd
  = do x <- osGetRadioBoxState hwnd; return (fromCBool x)
foreign import ccall osGetRadioBoxState :: WindowHandle -> IO CBool

setRadioBoxSelectState :: WindowHandle -> Bool -> IO ()
setRadioBoxSelectState hwnd x
  = osSetRadioBoxState hwnd (toCBool x)
foreign import ccall osSetRadioBoxState :: WindowHandle -> CBool -> IO ()

setRadioBoxGroup :: [WindowHandle] -> IO ()
setRadioBoxGroup handles = withArray0 nullHandle handles osSetRadioBoxGroup
foreign import ccall osSetRadioBoxGroup :: Ptr WindowHandle -> IO ()


-----------------------------------------------------------------------------------------
-- ListBox
-----------------------------------------------------------------------------------------
-- | Create a new list box. Takes an boolean argument that is 'True' when multiple
-- selection are allowed.
-- An event handler for list box clicks can be
-- installed with 'setControlCommandHandler'.
foreign import ccall "osCreateListBox" createListBox :: WindowHandle -> Bool -> IO WindowHandle

appendListBoxItem :: WindowHandle -> String -> IO ()
appendListBoxItem hwnd txt
  = withCString txt $ \ctxt -> osAppendListBoxItem hwnd ctxt
foreign import ccall osAppendListBoxItem :: WindowHandle -> CString -> IO ()

insertListBoxItem :: WindowHandle -> Int -> String -> IO ()
insertListBoxItem hwnd idx txt
  = withCString txt $ \ctxt ->
    osInsertListBoxItem hwnd (toCInt idx) ctxt
foreign import ccall osInsertListBoxItem :: WindowHandle -> CInt -> CString -> IO ()

removeListBoxItem :: WindowHandle -> Int -> IO ()
removeListBoxItem hwnd idx
  = osRemoveListBoxItem hwnd (toCInt idx)
foreign import ccall osRemoveListBoxItem :: WindowHandle -> CInt -> IO ()

foreign import ccall "osRemoveAllListBoxItems" removeAllListBoxItems :: WindowHandle -> IO ()

getListBoxRequestSize :: WindowHandle -> IO Size
getListBoxRequestSize hwnd = withCSizeResult (osGetListBoxReqSize hwnd)
foreign import ccall osGetListBoxReqSize :: WindowHandle -> Ptr CInt -> IO ()

getListBoxSingleSelection :: WindowHandle -> IO Int
getListBoxSingleSelection hwnd
  = do ci <- osGetListBoxSingleSelection hwnd
       return (fromCInt ci)
foreign import ccall osGetListBoxSingleSelection :: WindowHandle -> IO CInt

setListBoxSingleSelection :: WindowHandle -> Int -> IO ()
setListBoxSingleSelection hwnd i
  = osSetListBoxSingleSelection hwnd (toCInt i)
foreign import ccall osSetListBoxSingleSelection :: WindowHandle -> CInt -> IO ()

getListBoxItemSelectState :: WindowHandle -> Int -> IO Bool
getListBoxItemSelectState hwnd i
  = do cb <- osGetListBoxItemSelectState hwnd (toCInt i)
       return (fromCBool cb)
foreign import ccall osGetListBoxItemSelectState :: WindowHandle -> CInt -> IO CBool

setListBoxItemSelectState :: WindowHandle -> Int -> Bool -> IO ()
setListBoxItemSelectState hwnd i b
  = osSetListBoxItemSelectState hwnd (toCInt i) (toCBool b)
foreign import ccall osSetListBoxItemSelectState :: WindowHandle -> CInt -> CBool -> IO ()

foreign import ccall "osGetListBoxCurrentItem" getListBoxCurrentItem :: WindowHandle -> IO Int

-----------------------------------------------------------------------------------------
--  Popup
-----------------------------------------------------------------------------------------
-- | Create a  new popup box.
-- An event handler for pop up clicks can be
-- installed with 'setControlCommandHandler'.
foreign import ccall "osCreatePopUp" createPopUp :: WindowHandle -> IO WindowHandle

appendPopUpItem :: WindowHandle -> String -> IO ()
appendPopUpItem hwnd txt
  = withCString txt $ \ctxt -> osAppendPopUpItem hwnd ctxt
foreign import ccall osAppendPopUpItem :: WindowHandle -> CString -> IO ()

insertPopUpItem :: WindowHandle -> Int -> String -> IO ()
insertPopUpItem hwnd idx txt
  = withCString txt $ \ctxt ->
    osInsertPopUpItem hwnd (toCInt idx) ctxt
foreign import ccall osInsertPopUpItem :: WindowHandle -> CInt -> CString -> IO ()

removePopUpItem :: WindowHandle -> Int -> IO ()
removePopUpItem hwnd idx
  = osRemovePopUpItem hwnd (toCInt idx)
foreign import ccall osRemovePopUpItem :: WindowHandle -> CInt -> IO ()

foreign import ccall "osRemoveAllPopUpItems" removeAllPopUpItems :: WindowHandle -> IO ()

getPopUpRequestSize :: WindowHandle -> IO Size
getPopUpRequestSize hwnd = withCSizeResult (osGetPopUpReqSize hwnd)
foreign import ccall osGetPopUpReqSize :: WindowHandle -> Ptr CInt -> IO ()

getPopUpSelection :: WindowHandle -> IO Int
getPopUpSelection hwnd
  = do ci <- osGetPopUpSelection hwnd
       return (fromCInt ci)
foreign import ccall osGetPopUpSelection :: WindowHandle -> IO CInt

setPopUpSelection :: WindowHandle -> Int -> IO ()
setPopUpSelection hwnd i
  = osSetPopUpSelection hwnd (toCInt i)
foreign import ccall osSetPopUpSelection :: WindowHandle -> CInt -> IO ()

-----------------------------------------------------------------------------------------
-- Slider
-----------------------------------------------------------------------------------------

-- | Create a new horizontal slider control.
foreign import ccall "osCreateHorzSlider" createHorzSlider :: WindowHandle -> IO WindowHandle

-- | Create a new vertical slider control.
foreign import ccall "osCreateVertSlider" createVertSlider :: WindowHandle -> IO WindowHandle

getSliderRequestSize :: WindowHandle -> IO Size
getSliderRequestSize hwnd = withCSizeResult (osGetSliderReqSize hwnd)
foreign import ccall osGetSliderReqSize :: WindowHandle -> Ptr CInt -> IO ()

setSliderRange :: WindowHandle -> Int -> Int -> IO ()
setSliderRange hwnd min max = osSetSliderRange hwnd (toCInt min) (toCInt max)
foreign import ccall osSetSliderRange :: WindowHandle -> CInt -> CInt -> IO ()

getSliderRange :: WindowHandle -> IO (Int, Int)
getSliderRange hwnd = do
	ptr <- mallocArray 2
	pos <- osGetSliderRange hwnd ptr (ptr `advancePtr` 1)
	min <- peekElemOff ptr 0
	max <- peekElemOff ptr 1
	return (fromCInt min, fromCInt max)
foreign import ccall osGetSliderRange :: WindowHandle -> Ptr CInt -> Ptr CInt -> IO ()

setSliderPosition :: WindowHandle -> Int -> IO ()
setSliderPosition hwnd pos = osSetSliderPosition hwnd (toCInt pos)
foreign import ccall osSetSliderPosition :: WindowHandle -> CInt -> IO ()

getSliderPosition :: WindowHandle -> IO Int
getSliderPosition hwnd
  = do pos <- osGetSliderPosition hwnd
       return (fromCInt pos)
foreign import ccall osGetSliderPosition :: WindowHandle -> IO CInt

-----------------------------------------------------------------------------------------
-- ProgressBar
-----------------------------------------------------------------------------------------

-- | Create a new horizontal progress bar.
-- The boolean parameter specify whether the bar shows continuous or discrete values.
foreign import ccall "osCreateHorzProgressBar" createHorzProgressBar :: WindowHandle -> Bool -> IO WindowHandle

-- | Create a new vertical progress bar.
-- The boolean parameter specify whether the bar shows continuous or discrete values.
foreign import ccall "osCreateVertProgressBar" createVertProgressBar :: WindowHandle -> Bool -> IO WindowHandle

getProgressBarRequestSize :: WindowHandle -> IO Size
getProgressBarRequestSize hwnd = withCSizeResult (osGetProgressBarReqSize hwnd)
foreign import ccall osGetProgressBarReqSize :: WindowHandle -> Ptr CInt -> IO ()

setProgressBarFraction :: WindowHandle -> Int -> Int -> Int -> IO ()
setProgressBarFraction hwnd min max pos = osSetProgressBarFraction hwnd (toCInt min) (toCInt max) (toCInt pos)
foreign import ccall osSetProgressBarFraction :: WindowHandle -> CInt -> CInt -> CInt -> IO ()

getProgressBarFraction :: WindowHandle -> Int -> Int -> IO Int
getProgressBarFraction hwnd min max = do
	pos <- osGetProgressBarFraction hwnd (toCInt min) (toCInt max)
	return (fromCInt pos)
foreign import ccall osGetProgressBarFraction :: WindowHandle -> CInt -> CInt -> IO CInt

-----------------------------------------------------------------------------------------
-- CompoundControl
-----------------------------------------------------------------------------------------

-- | Create a new compound control
foreign import ccall "osCreateCompoundControl" createCompoundControl :: WindowHandle -> IO WindowHandle

getCompoundControlRequestSize :: WindowHandle -> IO Size
getCompoundControlRequestSize hwnd = withCSizeResult (osGetCompoundControlReqSize hwnd)
foreign import ccall osGetCompoundControlReqSize :: WindowHandle -> Ptr CInt -> IO ()

-----------------------------------------------------------------------------------------
-- GroupBox
-----------------------------------------------------------------------------------------

-- | Create a new group box
foreign import ccall "osCreateGroupBox" createGroupBox :: WindowHandle -> IO WindowHandle

getGroupBoxBordersSize :: WindowHandle -> IO (Int,Int,Int,Int)
getGroupBoxBordersSize hwnd = allocaArray 4 $ \cborders -> do
	osGetGroupBoxBordersSize hwnd cborders
	[l,t,r,b] <- peekArray 4 cborders
	return (fromCInt l,fromCInt t,fromCInt r,fromCInt b)
foreign import ccall osGetGroupBoxBordersSize :: WindowHandle -> Ptr CInt -> IO ()

getGroupBoxText :: WindowHandle -> IO String
getGroupBoxText hwnd = resultCString (osGetGroupBoxText hwnd)
foreign import ccall osGetGroupBoxText :: WindowHandle -> IO CString

setGroupBoxText :: WindowHandle -> String -> IO ()
setGroupBoxText hwnd txt = withCString txt (osSetGroupBoxText hwnd)
foreign import ccall osSetGroupBoxText :: WindowHandle -> CString -> IO ()

-----------------------------------------------------------------------------------------
-- Notebook
-----------------------------------------------------------------------------------------

-- | Create a new notebook control.
foreign import ccall "osCreateNotebook" createNotebook :: WindowHandle -> IO WindowHandle

getNotebookRequestSize :: WindowHandle -> IO Size
getNotebookRequestSize hwnd = withCSizeResult (osGetNotebookReqSize hwnd)
foreign import ccall osGetNotebookReqSize :: WindowHandle -> Ptr CInt -> IO ()

setNotebookLabelsPosition :: WindowHandle -> PositionType -> IO ()
setNotebookLabelsPosition hwnd position = osSetNotebookLabelsPosition hwnd (toCPositionType position)
foreign import ccall osSetNotebookLabelsPosition :: WindowHandle -> CInt -> IO ()

getNotebookLabelsPosition :: WindowHandle -> IO PositionType
getNotebookLabelsPosition hwnd = fmap fromCPositionType (osGetNotebookLabelsPosition hwnd)
foreign import ccall osGetNotebookLabelsPosition :: WindowHandle -> IO CInt

getNotebookSelection :: WindowHandle -> IO Int
getNotebookSelection hwnd
  = do ci <- osGetNotebookSelection hwnd
       return (fromCInt ci)
foreign import ccall osGetNotebookSelection :: WindowHandle -> IO CInt

setNotebookSelection :: WindowHandle -> Int -> IO ()
setNotebookSelection hwnd i
  = osSetNotebookSelection hwnd (toCInt i)
foreign import ccall osSetNotebookSelection :: WindowHandle -> CInt -> IO ()

foreign import ccall "osGetNotebookPageCount" getNotebookPageCount :: WindowHandle -> IO Int

insertNotebookPage :: WindowHandle -> Maybe Int -> IO WindowHandle
insertNotebookPage hwnd mb_pos = osInsertNotebookPage hwnd (fromMaybe (-1) mb_pos)
foreign import ccall osInsertNotebookPage :: WindowHandle -> Int -> IO WindowHandle

-- | Get the label of the notebook page.
getNotebookPageTitle :: WindowHandle -> IO String
getNotebookPageTitle hwnd = resultCString (osGetNotebookPageTitle hwnd)
foreign import ccall osGetNotebookPageTitle :: WindowHandle -> IO CString

-- | Set the label of the notebook page.
setNotebookPageTitle :: WindowHandle -> String -> IO ()
setNotebookPageTitle hwnd title = withCString title (osSetNotebookPageTitle hwnd)
foreign import ccall osSetNotebookPageTitle :: WindowHandle -> CString -> IO ()

foreign import ccall "osGetNotebookPagePos" getNotebookPagePos :: WindowHandle -> IO Int

foreign import ccall "osDestroyNotebookPage" destroyNotebookPage :: WindowHandle -> IO ()

getNotebookPageSize :: WindowHandle -> IO Size
getNotebookPageSize hwnd = withCSizeResult (osGetNotebookPageSize hwnd)
foreign import ccall osGetNotebookPageSize :: WindowHandle -> Ptr CInt -> IO ()

setNotebookPageBitmap :: WindowHandle -> Maybe Bitmap -> IO ()
setNotebookPageBitmap hwnd (Just bmp) = do
  map <- takeMVar windowBitmaps
  withCBitmap bmp (osSetNotebookPageBitmap hwnd)
  putMVar windowBitmaps (insert hwnd bmp map)
setNotebookPageBitmap hwnd Nothing    = do
  map <- takeMVar windowBitmaps
  osSetNotebookPageBitmap hwnd nullPtr
  putMVar windowBitmaps (delete hwnd map)
foreign import ccall osSetNotebookPageBitmap :: WindowHandle -> BitmapHandle -> IO ()

getNotebookPageBitmap :: WindowHandle -> IO (Maybe Bitmap)
getNotebookPageBitmap hwnd = do
  map <- readMVar windowBitmaps
  return (PtrMap.lookup hwnd map)
