{-# OPTIONS -fglasgow-exts -#include Types.h #-}
-----------------------------------------------------------------------------------------
{-| Module      :  Process
    Copyright   :  (c) Krasimir Angelov 2003
    License     :  BSD-style

    Maintainer  :  ka2_mail@yahoo.com
    Stability   :  provisional
    Portability :  portable
-}
-----------------------------------------------------------------------------------------
module Graphics.UI.Port.Process
	( start, quit, halt
	) where

import Graphics.UI.Port.Types
import Graphics.UI.Port.Window
import Graphics.UI.Port.Timer
import Graphics.UI.Port.Handlers
import Foreign.C
import System.Mem( performGC )
import Control.Monad(when)

-- | Start the event loop. Runs until 'quit' is called.
start :: String             -- ^ Application name
      -> String             -- ^ Application version
      -> DocumentInterface  -- ^ Document interface type for application
      -> IO a               -- ^ Startup action
      -> IO a
start name version di io = do
	(withCString name    $ \cname    ->
	 withCString version $ \cversion ->
	    osInit cname cversion (toCDocumentInterface di))
	setProcessDismissHandler (quit >> return ())
	r <- io
	osStart
	return r

-- | 'quit' exits the main event loop, closes any windows and menus, destroys all timers
-- and unregisters any event handlers. This function is automatically called when all
-- windows are closed.
quit :: IO Bool
quit = do
	r <- dismissAllWindows
	when r $ do
		destroyAllTimers
		osQuit
		performGC                -- to release any foreign objects
	return r

-- | 'quit' exits the main event loop, closes any windows and menus, destroys all timers
-- and unregisters any event handlers. This function is automatically called when all
-- windows are closed.
halt :: IO ()
halt = do
	destroyAllWindows
	destroyAllTimers
	osQuit
	performGC                -- to release any foreign objects

foreign import ccall "osInit"  osInit  :: CString -> CString -> CInt -> IO ()
foreign import ccall "osStart" osStart :: IO ()
foreign import ccall osQuit :: IO ()
