{-# OPTIONS -fffi -#include ToolBar.h #-}
-----------------------------------------------------------------------------------------
{-| Module      :  ToolBar
    Copyright   :  (c) Krasimir Angelov 2003
    License     :  BSD-style

    Maintainer  :  ka2_mail@yahoo.com
    Stability   :  provisional
    Portability :  portable

    ToolBar.
-}
-----------------------------------------------------------------------------------------
module Graphics.UI.Port.ToolBar
		(
		-- * Creation
		  createToolBar
		, destroyToolBar
		, getToolBarButtonCount

		, insertToolButton
		, insertToolCheckButton
		, insertToolRadioButton
		, insertToolDropDownButton
		, insertToolLine
		, setToolRadioGroup
		, setToolButtonBitmap,  getToolButtonBitmap
		, setToolButtonEnabled, getToolButtonEnabled
		, setToolButtonTip,     getToolButtonTip
		, setToolButtonText,    getToolButtonText
		, setToolButtonChecked, getToolButtonChecked
		, getToolItemPos
		, destroyToolItem
             	) where

import Graphics.UI.Port.Types
import Graphics.UI.Port.Handlers 
import Graphics.UI.Port.PtrMap as PtrMap
import Foreign
import Foreign.C
import Control.Concurrent.MVar
import Data.Maybe
import System.IO.Unsafe( unsafePerformIO )

createToolBar :: String -> PositionType -> Int -> Int -> Int -> IO WindowHandle
createToolBar name place band_num band_position offset =
	withCString name (\cname -> osCreateToolBar cname (toCPositionType place) band_num band_position offset)
foreign import ccall osCreateToolBar :: CString -> CInt -> Int -> Int -> Int -> IO WindowHandle

foreign import ccall "osDestroyToolBar" destroyToolBar :: WindowHandle -> IO ()

foreign import ccall "osGetToolBarButtonCount" getToolBarButtonCount :: WindowHandle -> IO Int

insertToolButton :: WindowHandle -> Maybe Int -> IO ToolHandle
insertToolButton toolBar pos = osInsertToolButton toolBar (fromMaybe (-1) pos)
foreign import ccall osInsertToolButton :: WindowHandle -> Int -> IO ToolHandle

insertToolCheckButton :: WindowHandle -> Maybe Int -> IO ToolHandle
insertToolCheckButton toolBar pos = osInsertToolCheckButton toolBar (fromMaybe (-1) pos)
foreign import ccall osInsertToolCheckButton :: WindowHandle -> Int -> IO ToolHandle

insertToolRadioButton :: WindowHandle -> Maybe Int -> IO ToolHandle
insertToolRadioButton toolBar pos = osInsertToolRadioButton toolBar (fromMaybe (-1) pos)
foreign import ccall osInsertToolRadioButton :: WindowHandle -> Int -> IO ToolHandle

setToolRadioGroup :: [ToolHandle] -> IO ()
setToolRadioGroup handles = withArray0 nullHandle handles osSetToolRadioGroup
foreign import ccall osSetToolRadioGroup :: Ptr ToolHandle -> IO ()

insertToolDropDownButton :: WindowHandle -> MenuHandle -> Maybe Int -> IO ToolHandle
insertToolDropDownButton toolBar menu pos = osInsertToolDropDownButton toolBar menu (fromMaybe (-1) pos)
foreign import ccall osInsertToolDropDownButton :: WindowHandle -> MenuHandle -> Int -> IO ToolHandle

insertToolLine :: WindowHandle -> Maybe Int -> IO ToolHandle
insertToolLine toolBar pos = osInsertToolLine toolBar (fromMaybe (-1) pos)
foreign import ccall osInsertToolLine :: WindowHandle -> Int -> IO ToolHandle

setToolButtonBitmap :: ToolHandle -> Maybe Bitmap -> IO ()
setToolButtonBitmap htool (Just bmp) = do
  map <- takeMVar toolBitmaps
  withCBitmap bmp (osSetToolButtonBitmap htool)
  putMVar toolBitmaps (insert htool bmp map)
setToolButtonBitmap htool Nothing    = do
  map <- takeMVar toolBitmaps
  osSetToolButtonBitmap htool nullPtr
  putMVar toolBitmaps (delete htool map)
foreign import ccall osSetToolButtonBitmap :: ToolHandle -> BitmapHandle -> IO ()

getToolButtonBitmap :: ToolHandle -> IO (Maybe Bitmap)
getToolButtonBitmap htool = do
  map <- readMVar toolBitmaps
  return (PtrMap.lookup htool map)

foreign import ccall "osSetToolButtonEnabled" setToolButtonEnabled :: ToolHandle -> Bool -> IO ()
foreign import ccall "osGetToolButtonEnabled" getToolButtonEnabled :: ToolHandle -> IO Bool

setToolButtonTip :: ToolHandle -> String -> IO ()
setToolButtonTip htool tip = withCString tip (osSetToolButtonTip htool)
foreign import ccall osSetToolButtonTip :: ToolHandle -> CString -> IO ()

getToolButtonTip :: ToolHandle -> IO String
getToolButtonTip htool = resultCString (osGetToolButtonTip htool)
foreign import ccall osGetToolButtonTip :: ToolHandle -> IO CString

setToolButtonText :: ToolHandle -> String -> IO ()
setToolButtonText htool tip = withCString tip (osSetToolButtonText htool)
foreign import ccall osSetToolButtonText :: ToolHandle -> CString -> IO ()

getToolButtonText :: ToolHandle -> IO String
getToolButtonText htool = resultCString (osGetToolButtonText htool)
foreign import ccall osGetToolButtonText :: ToolHandle -> IO CString

foreign import ccall "osSetToolButtonChecked" setToolButtonChecked :: ToolHandle -> Bool -> IO ()
foreign import ccall "osGetToolButtonChecked" getToolButtonChecked :: ToolHandle -> IO Bool

foreign import ccall "osGetToolItemPos" getToolItemPos :: ToolHandle -> IO Int

foreign import ccall "osDestroyToolItem" destroyToolItem :: ToolHandle -> IO ()
