#include "CommonDialogs.h"
#include "Internals.h"

char *osSelectDirectory(char *title, WindowHandle owner)
{
	GtkWidget *file_selector = gtk_file_selection_new(title);
	
	if (owner)
		owner = gtk_widget_get_toplevel(owner);
	else
		owner = gFrameWidget;
	gtk_window_set_transient_for(GTK_WINDOW(file_selector), GTK_WINDOW(owner));

	for (;;)
	{
		if (gtk_dialog_run(GTK_DIALOG(file_selector)) == GTK_RESPONSE_OK)
		{
			gchar *file_name;
			G_CONST_RETURN gchar *fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
			if (!g_file_test(fname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR))
			{
				GtkWidget *dialog =
						gtk_message_dialog_new (GTK_WINDOW(file_selector),
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									"%s directory not found",
									fname);
				 gtk_dialog_run (GTK_DIALOG (dialog));
				 gtk_widget_destroy (dialog);
				 continue;
			}

			file_name = g_strdup(fname);
			gtk_widget_destroy(file_selector);
			return file_name;
		}
		else
		{
			gtk_widget_destroy(file_selector);
			return NULL;
		}
	}
}

char *osSelectInputFile(char *title, char *filter, WindowHandle owner)
{
	GtkWidget *file_selector = gtk_file_selection_new(title);
	
	if (owner)
		owner = gtk_widget_get_toplevel(owner);
	else
		owner = gFrameWidget;
	gtk_window_set_transient_for(GTK_WINDOW(file_selector), GTK_WINDOW(owner));

	for (;;)
	{
		if (gtk_dialog_run(GTK_DIALOG(file_selector)) == GTK_RESPONSE_OK)
		{
			gchar *file_name;
			G_CONST_RETURN gchar *fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
			if (!g_file_test(fname, G_FILE_TEST_EXISTS))
			{
				GtkWidget *dialog =
						gtk_message_dialog_new (GTK_WINDOW(file_selector),
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									"%s file not found",
									fname);
				 gtk_dialog_run (GTK_DIALOG (dialog));
				 gtk_widget_destroy (dialog);
				 continue;
			}

			if (g_file_test(fname, G_FILE_TEST_IS_DIR))
			{
				GtkWidget *dialog =
						gtk_message_dialog_new (GTK_WINDOW(file_selector),
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_OK,
									"Path %s is a directory",
									fname);
				 gtk_dialog_run (GTK_DIALOG (dialog));
				 gtk_widget_destroy (dialog);
				 continue;
			}

			file_name = g_strdup(fname);
			gtk_widget_destroy(file_selector);
			return file_name;
		}
		else
		{
			gtk_widget_destroy(file_selector);
			return NULL;
		}
	}
}

char *osSelectInputFiles(char *title, char *filter, WindowHandle owner)
{
	GtkWidget *file_selector = gtk_file_selection_new(title);
	
	if (owner)
		owner = gtk_widget_get_toplevel(owner);
	else
		owner = gFrameWidget;
	gtk_window_set_transient_for(GTK_WINDOW(file_selector), GTK_WINDOW(owner));
	gtk_file_selection_set_select_multiple(GTK_FILE_SELECTION(file_selector), gtk_true());

	for (;;)
	{
		if (gtk_dialog_run(GTK_DIALOG(file_selector)) == GTK_RESPONSE_OK)
		{
			int pos_size;
			gchar *buffer;
			gchar **fname, **files = gtk_file_selection_get_selections(GTK_FILE_SELECTION(file_selector));
			
			pos_size = 1;
			for (fname = files; *fname; fname++)
			{
				if (!g_file_test(*fname, G_FILE_TEST_EXISTS))
				{
					GtkWidget *dialog =
							gtk_message_dialog_new (GTK_WINDOW(file_selector),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_MESSAGE_ERROR,
										GTK_BUTTONS_OK,
										"%s file not found",
										*fname);
					 gtk_dialog_run (GTK_DIALOG (dialog));
					 gtk_widget_destroy (dialog);
					 break;
				}

				if (g_file_test(*fname, G_FILE_TEST_IS_DIR))
				{
					GtkWidget *dialog =
							gtk_message_dialog_new (GTK_WINDOW(file_selector),
										GTK_DIALOG_DESTROY_WITH_PARENT,
										GTK_MESSAGE_ERROR,
										GTK_BUTTONS_OK,
										"Path %s is a directory",
										*fname);
					 gtk_dialog_run (GTK_DIALOG (dialog));
					 gtk_widget_destroy (dialog);
					 break;
				}
				
				pos_size += strlen(*fname)+1;
			}
			
			if (*fname)
			{
				g_strfreev(files);
				continue;
			}
			
			buffer   = malloc(pos_size);
			if (!buffer)
				return NULL;

			pos_size = 0;
			for (fname = files; *fname; fname++)
			{
				strcpy(buffer+pos_size,*fname);
				pos_size += strlen(*fname)+1;
			}
			
			gtk_widget_destroy(file_selector);
			g_strfreev(files);
			return buffer;
		}
		else
		{
			gtk_widget_destroy(file_selector);
			return NULL;
		}
	}
}

char *osSelectOutputFile(char *title, char *filter, char *nameptr, WindowHandle owner)
{
	GtkWidget *file_selector = gtk_file_selection_new(title);
	
	if (owner)
		owner = gtk_widget_get_toplevel(owner);
	else
		owner = gFrameWidget;
	gtk_window_set_transient_for(GTK_WINDOW(file_selector), GTK_WINDOW(owner));

	gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selector), nameptr);

	for (;;)
	{
		if (gtk_dialog_run(GTK_DIALOG(file_selector)) == GTK_RESPONSE_OK)
		{
			gchar *file_name;
			G_CONST_RETURN gchar *fname = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
			if (g_file_test(fname, G_FILE_TEST_EXISTS))
			{
				gint res;
				GtkWidget *dialog =
						gtk_message_dialog_new (GTK_WINDOW(file_selector),
									GTK_DIALOG_DESTROY_WITH_PARENT,
									GTK_MESSAGE_WARNING,
									GTK_BUTTONS_YES_NO,
									"%s already exists. Do you want to replace id?",
									fname);
				 res = gtk_dialog_run (GTK_DIALOG (dialog));
				 gtk_widget_destroy (dialog);

				 if (res == GTK_RESPONSE_NO) continue;
			}

			file_name = g_strdup(fname);
			gtk_widget_destroy(file_selector);
			return file_name;
		}
		else
		{
			gtk_widget_destroy(file_selector);
			return NULL;
		}
	}
}
