#include "Menu.h"
#include "Window.h"
#include "Internals.h"
#include "Handlers_stub.h"
#include <gdk/gdkkeysyms.h>

static MenuHandle getMenu(MenuHandle parent)
{
	if (!parent)
	{
		if (!gMenuBar)
		{
			gMenuBar = gtk_menu_bar_new();
			gtk_widget_show(gMenuBar);
			gnome_app_set_menus(GNOME_APP(gFrameWidget), GTK_MENU_BAR(gMenuBar));
		}

		parent = gMenuBar;
	}
	else
		if (GTK_IS_MENU_ITEM(parent))
			parent = gtk_menu_item_get_submenu(GTK_MENU_ITEM(parent));

	return parent;
};

static int recalcPos(GtkWidget *menu, int pos)
{
	int new_pos;
	GList *children;

	children = GTK_MENU_SHELL(menu)->children;

	if (pos >= 0)
	{
		new_pos = 0;
		while (children && pos > 0)
		{
			pos--;
			new_pos++;
			children = children->next;
		}

		if (!children)
			new_pos = (gDocumentInterface == 1 || menu != gMenuBar) ? -1 : new_pos-1;
	}
	else
	{
		new_pos = (gDocumentInterface == 1 || menu != gMenuBar) ? -1 : g_list_length(children)-1;
	}

	return new_pos;
}

static void menu_show(GtkWidget *widget, gpointer user_data)
{
	GtkWidget *menuItem;
	GList *child = GTK_MENU_SHELL(widget)->children;

	while (child)
	{
		menuItem = (GtkWidget *) child->data;

		if (GTK_IS_IMAGE_MENU_ITEM(menuItem) ||
		    GTK_IS_CHECK_MENU_ITEM(menuItem) ||
		    GTK_IS_RADIO_MENU_ITEM(menuItem))
		{
			handleMenuUpdate(menuItem);
		}

		child = child->next;
	}
}

MenuHandle osCreatePopupMenu()
{
	GtkWidget *popUpMenu = gtk_menu_new();
	gtk_signal_connect (GTK_OBJECT(popUpMenu), "destroy",
		GTK_SIGNAL_FUNC(handleMenuDestroy),
		NULL);
	gtk_signal_connect (GTK_OBJECT(popUpMenu), "show",
		GTK_SIGNAL_FUNC(menu_show),
		NULL);
	return popUpMenu;
}

struct PositioningParams
{
	int x, y;
	GtkWidget *widget;
};

static void menu_positioning(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, struct PositioningParams *params)
{
	gdk_window_get_origin(params->widget->window, x, y);
	*x += params->x;
	*y += params->y;

	*push_in = TRUE;
	free(params);
}

void osTrackPopupMenu(MenuHandle handle, WindowHandle hwnd, int x, int y)
{
	struct PositioningParams *params = malloc(2*sizeof(struct PositioningParams));
	if (params)
	{
		params->x = x;
		params->y = y;
		params->widget = hwnd;
		gtk_menu_popup(GTK_MENU(handle), NULL, NULL, menu_positioning, params, 0, gtk_get_current_event_time());
	}
}

MenuHandle osInsertMenu(MenuHandle parent, int pos)
{
	GtkWidget *menu;
	GtkWidget *menuItem = gtk_menu_item_new_with_mnemonic("");
	GtkWidget *popUpMenu = gtk_menu_new();
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), popUpMenu);

	gtk_signal_connect (GTK_OBJECT(menuItem), "destroy",
		GTK_SIGNAL_FUNC(handleMenuDestroy),
		NULL);
	gtk_signal_connect (GTK_OBJECT(popUpMenu), "show",
		GTK_SIGNAL_FUNC(menu_show),
		NULL);

	menu = getMenu(parent);
	pos = recalcPos(menu, pos);

	gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItem, pos);
	gtk_widget_show_all(menuItem);
	return menuItem;
};

MenuHandle osInsertMenuItem(MenuHandle parent, int pos)
{
	GtkWidget *menu;
	GtkWidget *label, *menuItem;

	menuItem = gtk_image_menu_item_new();
	label = gtk_accel_label_new("");
	gtk_container_add(GTK_CONTAINER(menuItem), label);
	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(label), menuItem);

	gtk_signal_connect (GTK_OBJECT(menuItem), "activate",
			GTK_SIGNAL_FUNC(handleMenuCommand),
			NULL);
	gtk_signal_connect (GTK_OBJECT(menuItem), "destroy",
			GTK_SIGNAL_FUNC(handleMenuDestroy),
			NULL);

	menu = getMenu(parent);
	pos = recalcPos(menu, pos);

	gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItem, pos);
	gtk_widget_show_all(menuItem);
	return menuItem;
};

MenuHandle osInsertMenuCheckItem(MenuHandle parent, int pos)
{
	GtkWidget *menu;
	GtkWidget *label, *menuItem;

	menuItem = gtk_check_menu_item_new();
	label = gtk_accel_label_new("");
	gtk_container_add(GTK_CONTAINER(menuItem), label);
	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(label), menuItem);

	gtk_signal_connect (GTK_OBJECT(menuItem), "activate",
			GTK_SIGNAL_FUNC(handleMenuCommand),
			NULL);
	gtk_signal_connect (GTK_OBJECT(menuItem), "destroy",
			GTK_SIGNAL_FUNC(handleMenuDestroy),
			NULL);

	menu = getMenu(parent);
	pos = recalcPos(menu, pos);

	gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItem, pos);
	gtk_widget_show_all(menuItem);
	return menuItem;
};

MenuHandle osInsertMenuSeparatorItem(MenuHandle parent, int pos)
{
	GtkWidget *menu;
	GtkWidget *item =  gtk_separator_menu_item_new();
	gtk_signal_connect (GTK_OBJECT(item), "destroy",
			GTK_SIGNAL_FUNC(handleMenuDestroy),
			NULL);

	menu = getMenu(parent);
	pos = recalcPos(menu, pos);

	gtk_menu_shell_insert(GTK_MENU_SHELL(getMenu(parent)), item, pos);
	gtk_widget_show(item);
	return item;
}

static void handleRadioMenuCommand(GtkMenuItem *menuitem)
{
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)))
		handleMenuCommand(menuitem);
}

MenuHandle osInsertMenuRadioItem(MenuHandle parent, int pos)
{
	GtkWidget *menu;
	GtkWidget *menuItem, *label;

	menuItem = gtk_radio_menu_item_new(NULL);
	GTK_CHECK_MENU_ITEM (menuItem)->active = FALSE;
	label = gtk_accel_label_new("");
	gtk_container_add(GTK_CONTAINER(menuItem), label);
	gtk_accel_label_set_accel_widget(GTK_ACCEL_LABEL(label), menuItem);

	gtk_signal_connect (GTK_OBJECT(menuItem), "activate",
		GTK_SIGNAL_FUNC(handleRadioMenuCommand),
		NULL);
	gtk_signal_connect (GTK_OBJECT(menuItem), "destroy",
		GTK_SIGNAL_FUNC(handleMenuDestroy),
		NULL);

	menu = getMenu(parent);
	pos = recalcPos(menu, pos);

	gtk_menu_shell_insert(GTK_MENU_SHELL(menu), menuItem, pos);
	gtk_widget_show_all(menuItem);
	return menuItem;
}

void osSetMenuRadioGroup(MenuHandle *handles)
{
	MenuHandle *phandle;
	GSList *group = NULL;

	for (phandle = handles; *phandle; phandle++)
	{
		gtk_radio_menu_item_set_group(GTK_RADIO_MENU_ITEM(*phandle), group);
		group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(*phandle));
	}
}

void osDestroyMenu(MenuHandle handle)
{
	gtk_widget_destroy(handle);
}

int osGetMenuItemCount(MenuHandle handle)
{
	int count;
	GList *children;

	count = 0;
	children = GTK_MENU_SHELL(getMenu(handle))->children;
	while (children)
	{
		count++;
		children = children->next;
	}

	return count;
}

void osSetMenuItemEnabled(MenuHandle item, BOOL bState)
{
	if (GTK_IS_MENU_ITEM(item))
		gtk_widget_set_sensitive(item,bState);
};

BOOL osGetMenuItemEnabled(MenuHandle item)
{
	if (GTK_IS_MENU_ITEM(item))
		return GTK_WIDGET_SENSITIVE(item);
	else
		return TRUE;
};

void osSetMenuItemChecked(MenuHandle item, BOOL bState)
{
	gtk_check_menu_item_set_active  (GTK_CHECK_MENU_ITEM(item), bState);
};

BOOL osGetMenuItemChecked(MenuHandle item)
{
	return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
};

void osSetMenuLabel(MenuHandle item, char* title)
{
	if (GTK_IS_MENU_ITEM(item))
	{
		gchar *szText = toMnemonicString(title);
		gtk_label_set_text_with_mnemonic(GTK_LABEL(GTK_BIN(item)->child), szText);
		rfree(szText);
	}
}

char *osGetMenuLabel(MenuHandle item)
{
	if (GTK_IS_MENU_ITEM(item))
		return fromMnemonicString(gtk_label_get_text(GTK_LABEL(GTK_BIN(item)->child)));
	else
		return NULL;
};

static gboolean find_accel_by_widget(GtkAccelKey *key, GClosure *closure, gpointer data)
{
	return (closure->data == data);
}

void osSetMenuItemAccel(MenuHandle item, int key, unsigned int mods)
{
	GdkModifierType modifier;
	GtkAccelKey *accel_key;
	GtkAccelGroup *accel_group = (GtkAccelGroup *) gtk_accel_groups_from_object (G_OBJECT(gFrameWidget))->data;

	accel_key = gtk_accel_group_find(accel_group, find_accel_by_widget, item);
	if (accel_key) gtk_widget_remove_accelerator(item, accel_group, accel_key->accel_key, accel_key->accel_mods);

	if (!key) return;

	modifier = 0;
	if (mods & shiftBIT) modifier |= GDK_SHIFT_MASK;
	if (mods & ctrlBIT)  modifier |= GDK_CONTROL_MASK;
	if (mods & altBIT)   modifier |= GDK_MOD1_MASK;

	switch (key)
	{
	case kbBackSpace: key = GDK_BackSpace; break;
	case kbTab:       key = GDK_Tab;       break;
	case kbEnter:     key = GDK_Return;    break;
	case kbEscape:    key = GDK_Escape;    break;
	case kbBegin:     key = GDK_Begin;     break;
	case kbClear:	  key = GDK_Clear;     break;
	case kbDelete:    key = GDK_Delete;    break;
	case kbDown:      key = GDK_Down;      break;
	case kbEnd:       key = GDK_End;       break;
	case kbF1:        key = GDK_F1;        break;
	case kbF2:        key = GDK_F2;        break;
	case kbF3:        key = GDK_F3;        break;
	case kbF4:        key = GDK_F4;        break;
	case kbF5:        key = GDK_F5;        break;
	case kbF6:        key = GDK_F6;        break;
	case kbF7:        key = GDK_F7;        break;
	case kbF8:        key = GDK_F8;        break;
	case kbF9:        key = GDK_F9;        break;
	case kbF10:       key = GDK_F10;       break;
	case kbF11:       key = GDK_F11;       break;
	case kbF12:       key = GDK_F12;       break;
	case kbF13:       key = GDK_F13;       break;
	case kbF14:       key = GDK_F14;       break;
	case kbF15:       key = GDK_F15;       break;
	case kbHelp:      key = GDK_Help;      break;
	case kbLeft:      key = GDK_Left;      break;
	case kbPgDown:    key = GDK_Page_Down; break;
	case kbPgUp:      key = GDK_Page_Up;   break;
	case kbRight:     key = GDK_Right;     break;
	case kbUp:        key = GDK_Up;        break;
	default:
		if (key >= 256)
		{
			key -= 256;
			modifier = GDK_MOD1_MASK;
		}

	  	if (iscntrl(key))
		{
			key += '@';
			modifier |= GDK_CONTROL_MASK;
		}
		else
			if (isupper(key))
				modifier |= GDK_SHIFT_MASK;
			else
				key = toupper(key);
		break;
	}

	gtk_widget_add_accelerator(item, "activate",
				   accel_group,
				   key,
				   modifier,
				   GTK_ACCEL_VISIBLE);
};

void osGetMenuItemAccel(MenuHandle item, int *key, unsigned int *mods)
{
	GtkAccelKey *accel_key;
	GtkAccelGroup *accel_group = (GtkAccelGroup *) gtk_accel_groups_from_object (G_OBJECT(gFrameWidget))->data;

	accel_key = gtk_accel_group_find(accel_group, find_accel_by_widget, item);

	if (accel_key)
	{
	        *mods = 0;
	        if (accel_key->accel_mods & GDK_SHIFT_MASK)   *mods |= shiftBIT;
	        if (accel_key->accel_mods & GDK_CONTROL_MASK) *mods |= ctrlBIT;
	        if (accel_key->accel_mods & GDK_MOD1_MASK)    *mods |= altBIT;

		switch (accel_key->accel_key)
		{
		case GDK_Up:       *key = kbUp;        break;
		case GDK_Down:     *key = kbDown;      break;
		case GDK_Left:     *key = kbLeft;      break;
		case GDK_Right:    *key = kbRight;     break;
		case GDK_Page_Up:  *key = kbPgUp;      break;
		case GDK_Page_Down:*key = kbPgDown;    break;
		case GDK_End:      *key = kbEnd;       break;
		case GDK_Begin:    *key = kbBegin;     break;
		case GDK_BackSpace:*key = kbBackSpace; break;
		case GDK_Delete:   *key = kbDelete;    break;
		case GDK_Tab:      *key = kbTab;       break;
		case GDK_Return:   *key = kbEnter;     break;
		case GDK_Escape:   *key = kbEscape;    break;
		case GDK_Help:     *key = kbHelp;      break;
		case GDK_F1:       *key = kbF1;        break;
		case GDK_F2:       *key = kbF2;        break;
		case GDK_F3:       *key = kbF3;        break;
		case GDK_F4:       *key = kbF4;        break;
		case GDK_F5:       *key = kbF5;        break;
		case GDK_F6:       *key = kbF6;        break;
		case GDK_F7:       *key = kbF7;        break;
		case GDK_F8:       *key = kbF8;        break;
		case GDK_F9:       *key = kbF9;        break;
		case GDK_F10:      *key = kbF10;       break;
		case GDK_F11:      *key = kbF11;       break;
		case GDK_F12:      *key = kbF12;       break;
		case GDK_Clear:    *key = kbClear;     break;
		default:
		        *mods = 0;

			*key = accel_key->accel_key;
			if (accel_key->accel_mods & GDK_CONTROL_MASK)
			  *key = toupper(accel_key->accel_key) - '@';
			else
			  if (accel_key->accel_mods & GDK_SHIFT_MASK)
			    *key = toupper(accel_key->accel_key);

			if (accel_key->accel_mods & GDK_MOD1_MASK)
			  *key += 256;
			break;
		}
	}
	else
	{
		*key  = 0;
		*mods = 0;
	}
};

void osSetMenuItemBitmap(MenuHandle item, BitmapHandle bitmap)
{
	GtkWidget *image = gtk_image_menu_item_get_image(GTK_IMAGE_MENU_ITEM(item));

	if (bitmap)
	{
		if (!image)
		{
			image = gtk_image_new_from_pixbuf(bitmap->pixbuf);
			gtk_widget_show(image);
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), image);
		}
		else
 			gtk_image_set_from_pixbuf(GTK_IMAGE(image),bitmap->pixbuf);
	}
	else
	{
		if (image)
		{
			gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(item), NULL);
		}
	}
};

int osGetMenuItemPos(MenuHandle handle)
{
	int pos;
	GList *children;

	pos = 0;
	children = GTK_MENU_SHELL(getMenu(handle))->children;
	while (children && children->data != handle)
	{
		pos++;
		children = children->next;
	}

	return (children ? pos : -1);
}
