#include "Types.h"
#include "Internals.h"
#include "DockBar.h"
#include "Handlers_stub.h"


typedef struct
{
	int nSize;
	int nOffset;
	HWND hWnd;
} BarData;
	
typedef struct
{
	int nSize;
	int nOffset;
	int nBarCount;
	BarData *pBars;
} BandData;

typedef struct
{
	int nBandCount;
	BandData *pBands;
} DockBarData;

static void RecalcDockBarLayout(DockBarData *pData, BOOL bVert)
{
	int i, j;
	BarData *pBar;
	BandData *pBand;
	int nBarOffset, nBandOffset;
	SIZE sz;
	
	nBandOffset = 0;
	for (i = 0; i < pData->nBandCount; i++)
	{
		pBand = &pData->pBands[i];
		
		pBand->nSize = 0;
		pBand->nOffset = nBandOffset;
		
		nBarOffset = 0;
		for (j = 0; j < pBand->nBarCount; j++)
		{
			pBar = &pBand->pBars[j];
			GetToolBarSize(pBar->hWnd, &sz);
			
			pBar->nOffset = max(pBar->nOffset, nBarOffset);
			if (bVert)
			{
				pBand->nSize = max(pBand->nSize, sz.cx);
				pBar->nSize = sz.cy;
				
				SetWindowPos(pBar->hWnd,NULL,pBand->nOffset,pBar->nOffset,sz.cx,sz.cy,SWP_NOZORDER);
			}
			else
			{
				pBand->nSize = max(pBand->nSize, sz.cy);
				pBar->nSize = sz.cx;
				
				SetWindowPos(pBar->hWnd,NULL,pBar->nOffset,pBand->nOffset,sz.cx,sz.cy,SWP_NOZORDER);
			}
			
			nBarOffset = pBar->nOffset + pBar->nSize;
		}
	
		nBandOffset = pBand->nOffset + pBand->nSize;
	}
}

static int GetDockBarSize(HWND hDockBar)
{
    int i, nSize;
    DockBarData *pData;
		
	nSize = 0;
	pData = (DockBarData *) GetWindowLong(hDockBar, GWL_USERDATA);
	
	RecalcDockBarLayout(pData, GetWindowLong(hDockBar, GWL_STYLE) & CCS_VERT);
	
	for (i = 0; i < pData->nBandCount; i++)
		nSize += pData->pBands[i].nSize;

	return nSize;
}

void RelayoutFrameBars()
{
	RECT rect;
	FrameData *pData;
	int nWidth, nHeight, nLeft, nTop, nRight, nBottom;

	if (!ghWndFrame)
		return NULL;

	pData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	
	GetClientRect(ghWndFrame, &rect);
	
	nWidth  = rect.right-rect.left;
	nHeight = rect.bottom-rect.top;
	nLeft   = GetDockBarSize(pData->hLeftBar);
	nTop    = GetDockBarSize(pData->hTopBar);
	nRight  = GetDockBarSize(pData->hRightBar);
	nBottom = GetDockBarSize(pData->hBottomBar);

	SetWindowPos(pData->hLeftBar,  NULL,0,nTop,nLeft,nHeight-(nBottom+nTop),SWP_NOZORDER);
	SetWindowPos(pData->hTopBar,   NULL,0,0,nWidth,nTop,SWP_NOZORDER);
	SetWindowPos(pData->hRightBar, NULL,nWidth-nRight,nTop,nRight,nHeight-(nBottom+nTop),SWP_NOZORDER);
	SetWindowPos(pData->hBottomBar,NULL,0,nHeight-nBottom,nWidth,nBottom,SWP_NOZORDER);

	SetWindowPos(pData->hClientWnd,NULL,nLeft,nTop,nWidth-(nRight+nLeft),nHeight-(nBottom+nTop),SWP_NOZORDER);
}

extern void osActivateToolItem(ToolHandle toolButton);

LRESULT CALLBACK HDockBarFunction(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	DockBarData *pData = (DockBarData *) GetWindowLong(hWnd, GWL_USERDATA);
			
	switch (uMsg)
	{
	case WM_CREATE:
		{
			HMENU hMenu;
			
			if (((CREATESTRUCT *) lParam)->style & WS_CHILD)
			{
				pData = malloc(sizeof(DockBarData));
				if (!pData) return -1;

				pData->nBandCount = 0;
				pData->pBands     = NULL;

				SetWindowLong(hWnd, GWL_USERDATA, (LONG) pData);
			}

			hMenu = GetSystemMenu(hWnd, FALSE);
			if (hMenu)
			{
				DeleteMenu(hMenu, SC_RESTORE,  MF_BYCOMMAND);
				DeleteMenu(hMenu, SC_MINIMIZE, MF_BYCOMMAND);
				DeleteMenu(hMenu, SC_MAXIMIZE, MF_BYCOMMAND);
				DeleteMenu(hMenu, SC_CLOSE,    MF_BYCOMMAND);
				AppendMenu(hMenu, MF_STRING, SC_CLOSE, "Hide");
			}
		}
		break;
	case WM_DESTROY:
		if (pData)
		{
			int i;
			
			SetWindowLong(hWnd, GWL_USERDATA, 0);
			for (i = 0; i < pData->nBandCount; i++)
				free(pData->pBands[i].pBars);
			free(pData->pBands);
			free(pData);
		}
		break;
	case WM_CLOSE:
		ShowWindow(hWnd, SW_HIDE);
		return 0;
	case WM_COMMAND:
		{
			ToolHandle handle;
			TBBUTTONINFO tbbi;
			HWND hToolBar = (HWND) lParam;
		
			tbbi.cbSize = sizeof(tbbi);
			tbbi.dwMask = TBIF_LPARAM;
			tbbi.lParam = 0;
			SendMessage(hToolBar, TB_GETBUTTONINFO, LOWORD(wParam), (LPARAM) &tbbi);
			
			handle = (ToolHandle) tbbi.lParam;
			if (handle) osActivateToolItem(handle);
		}
		break;
	case WM_NOTIFY:
		{
			ToolHandle handle;
			TBBUTTONINFO tbbi;
			LPNMTOOLBAR lpNMToolBar = (LPNMTOOLBAR) lParam;
			
			if (lpNMToolBar->hdr.code == TBN_DROPDOWN)
			{			
				tbbi.cbSize = sizeof(tbbi);
				tbbi.dwMask = TBIF_LPARAM;
				tbbi.lParam = 0;
				SendMessage(lpNMToolBar->hdr.hwndFrom, TB_GETBUTTONINFO, lpNMToolBar->iItem, (LPARAM) &tbbi);
				
				handle = (ToolHandle) tbbi.lParam;
				if (handle)
					osActivateToolItem(handle);
			}
		}
		break;
	case WM_SIZE:
		if (!pData)
		{
			int nWidth  = LOWORD(lParam);
            int nHeight = HIWORD(lParam);
            
        	SetWindowPos(GetWindow(hWnd, GW_CHILD),NULL,0,0,nWidth,nHeight,SWP_NOZORDER);
		}
		break;
	case WM_NCLBUTTONDOWN:
		if (wParam == HTCAPTION)
			SendMessage(GetWindow(hWnd, GW_CHILD), uMsg, wParam, lParam);
		break;
	}
	
	return DefWindowProc(hWnd, uMsg, wParam, lParam);
}

void DockToolBarToRect(HWND hDockBar, HWND hWnd, RECT rect)
{
	int nBandNum;
	int nPosition;
	int nOffset;
	int nCentralOffset;
	DockBarData *pData = (DockBarData *) GetWindowLong(hDockBar, GWL_USERDATA);	
	BOOL bVert = GetWindowLong(hDockBar, GWL_STYLE) & CCS_VERT;
	BandData *pBand;
	
	RecalcDockBarLayout(pData, bVert);
	
	if (bVert)
	{
		nOffset = rect.top;
		nCentralOffset = (rect.right-rect.left)/2;
	}
	else
	{
		nOffset = rect.left;
		nCentralOffset = (rect.bottom-rect.top)/2;
	}
	
	for (nBandNum = 0; nBandNum < pData->nBandCount; nBandNum++)
	{
		if (pData->pBands[nBandNum].nOffset <= nCentralOffset &&
		    nCentralOffset <= pData->pBands[nBandNum].nOffset+pData->pBands[nBandNum].nSize)
			break;
	}
	
	if (nBandNum >= pData->nBandCount)
		nPosition = 0;
	else
	{
		pBand = &pData->pBands[nBandNum];
		
		if (nOffset < pBand->pBars[0].nOffset)
			nPosition = 0;
		else
		{
			for (nPosition = 1; nPosition < pBand->nBarCount; nPosition++)
			{
				if (nOffset < pBand->pBars[nPosition].nOffset)
					break;
			}
		}
	}
	
	DockToolBar(hDockBar, hWnd, nBandNum, nPosition, nOffset);
}

void DockToolBar(HWND hDockBar, HWND hWnd, int nBandNum, int nPosition, int nOffset)
{
	BarData *pBar;
	BandData *pBand;
	DockBarData *pData = (DockBarData *) GetWindowLong(hDockBar, GWL_USERDATA);
	
	SetParent(hWnd, hDockBar);
	
	if (pData)
	{
		if (nBandNum >= pData->nBandCount)
		{
			pData->pBands = realloc(pData->pBands, (pData->nBandCount+1)*sizeof(BandData));
			memset(pData->pBands+pData->nBandCount, 0, sizeof(BandData));
			nBandNum = pData->nBandCount;
			pData->nBandCount++;
		}
		pBand = &pData->pBands[nBandNum];

		if (nPosition >= pBand->nBarCount)
			nPosition = pBand->nBarCount;
		pBand->pBars = realloc(pBand->pBars, (pBand->nBarCount+1)*sizeof(BarData));
		memmove(pBand->pBars+nPosition+1, pBand->pBars+nPosition, (pBand->nBarCount-nPosition)*sizeof(BarData));	
		pBand->nBarCount++;

		pBar = &pBand->pBars[nPosition];
		memset(pBar, 0, sizeof(BarData));
		pBar->nOffset = nOffset;
		pBar->hWnd = hWnd;

		RecalcDockBarLayout(pData, GetWindowLong(hWnd, GWL_STYLE) & CCS_VERT);
	}
}

void UndockToolBar(HWND hDockBar, HWND hWnd)
{
	int i, j;
	BarData *pBar;
	BandData *pBand;
	DockBarData *pData = (DockBarData *) GetWindowLong(hDockBar, GWL_USERDATA);

	SetParent(hWnd, NULL);
	
	if (pData)
	{	
		for (i = 0; i < pData->nBandCount; i++)
		{
			pBand = &pData->pBands[i];

			for (j = 0; j < pBand->nBarCount; j++)
			{
				pBar = &pBand->pBars[j];

				if (pBar->hWnd == hWnd)
				{
					memmove(pBand->pBars+j, pBand->pBars+j+1, (pBand->nBarCount-j-1)*sizeof(BarData));
					pBand->pBars = realloc(pBand->pBars, (pBand->nBarCount-1)*sizeof(BarData));
					pBand->nBarCount--;

					if (pBand->nBarCount == 0)
					{
						memmove(pData->pBands+i, pData->pBands+i+1, (pData->nBandCount-i-1)*sizeof(BandData));
						pData->pBands = realloc(pData->pBands, (pData->nBandCount-1)*sizeof(BandData));
						pData->nBandCount--;
					}

					return;
				}
			}
		}
	}
	else
	{
		DestroyWindow(hDockBar);
	}
}
