#include "Types.h"
#include "Window.h"
#include "Menu.h"
#include "DockBar.h"
#include "MenuHandlesMap.h"
#include "Canvas.h"
#include "Internals.h"
#include "Handlers_stub.h"

#define OSMenuIDEnd 1500

#define Spacing 1

void DrawMenuText(HDC hdc, LPTSTR lpchText, LPRECT lpRect)
{
	int n = 0;
	while (lpchText[n] && lpchText[n] != '\t') n++;
	
	DrawText(hdc, lpchText,      n, lpRect, DT_SINGLELINE|DT_VCENTER|DT_LEFT);
	DrawText(hdc, lpchText+n+1, -1, lpRect, DT_SINGLELINE|DT_VCENTER|DT_RIGHT);
}

LRESULT CALLBACK HFrameSharedFunction(int DocumentInterface, HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int pos;
	HMENU hParent;
	MENUITEMINFO mii;
	MenuHandle handle;
	FrameData *pData = (FrameData *) GetWindowLong(hWnd,GWL_USERDATA);

	switch (uMsg)
	{
	case WM_CLOSE:
		handleProcessDismiss();
		return 0;
	case WM_DESTROY:
	    notifyAllHandlesForDestroy(pData->pMenuHandlesMap);
		handleProcessDestroy();
		free(pData->lpszAppName);
		free(pData->lpszAppVersion);
		deleteMenuHandlesMap(pData->pMenuHandlesMap);
		free(pData);
		PostQuitMessage(0);
		break;
	case WM_CREATE:
		{
			int nLen;

			pData = (FrameData *) malloc(sizeof(FrameData));
			if (!pData) return -1;

			pData->hClientWnd = NULL;
			pData->DocumentInterface = DocumentInterface;
			pData->lpszAppName = NULL;
			pData->lpszAppVersion = NULL;
			pData->pMenuHandlesMap = newMenuHandlesMap();

			nLen = GetWindowTextLength(hWnd);
			if (nLen > 0)
			{
				pData->lpszAppName = malloc(nLen+1);
				if (!pData->lpszAppName)
				{
					free(pData);
					return -1;
				}
				GetWindowText(hWnd,pData->lpszAppName,nLen+1);
			}
			
			pData->hLeftBar  = CreateWindow("HDOCKBAR",
											NULL,
											WS_CHILD | WS_VISIBLE | CCS_VERT,
											0,0,0,0,
											hWnd,
											NULL,
											(HANDLE) ghModule,
											NULL);
			pData->hTopBar   = CreateWindow("HDOCKBAR",
											NULL,
											WS_CHILD | WS_VISIBLE,
											0,0,0,0,
											hWnd,
											NULL,
											(HANDLE) ghModule,
											NULL);
			pData->hRightBar = CreateWindow("HDOCKBAR",
											NULL,
											WS_CHILD | WS_VISIBLE | CCS_VERT,
											0,0,0,0,
											hWnd,
											NULL,
											(HANDLE) ghModule,
											NULL);
			pData->hBottomBar= CreateWindow("HDOCKBAR",
											NULL,
											WS_CHILD | WS_VISIBLE,
											0,0,0,0,
											hWnd,
											NULL,
											(HANDLE) ghModule,
											NULL);
				
			SetWindowLong(hWnd,GWL_USERDATA,(LONG) pData);
		}
		break;
	case WM_SIZE:
		RelayoutFrameBars();
		break;
	case WM_COMMAND:
		if (lParam == 0)
		{
			handle = getMenuHandle(pData->pMenuHandlesMap, (UINT) LOWORD(wParam));
			if (handle)
			{
				if (handle->type & (MENU_RADIO_ITEM | MENU_CHECK_ITEM))
					osSetMenuItemChecked(handle, TRUE);
				else
					handleMenuCommand(handle);				
			}
		}
        break;
    case WM_MEASUREITEM:
        {
            LPMEASUREITEMSTRUCT lpMIS = (LPMEASUREITEMSTRUCT) lParam;
            
            if (lpMIS->CtlType == ODT_MENU)
            {
				handle = getMenuHandle(pData->pMenuHandlesMap, lpMIS->itemID);
				
				if (handle && handle->bitmap)
				{
            		NONCLIENTMETRICS ncm;
            		HFONT hFont, hOldFont;
            		RECT rc;
            		HDC hDC;
            		long lDims;
            		WORD wCheckWidth, wCheckHeight;
            	
            		hDC = GetDC(hWnd);
						
					ZeroMemory(&ncm,sizeof(ncm));
					ncm.cbSize = sizeof(ncm);
				
					// Get the menu dimensions
					SystemParametersInfo(SPI_GETNONCLIENTMETRICS,0,(PVOID)&ncm,FALSE);
						
					// Create a font based on menu metrics				
					hFont = CreateFontIndirect(&ncm.lfMenuFont);

					hOldFont = SelectObject(hDC, hFont);

					// Draw out menu item caption - text.					
					pos = getMenuPos(pData->pMenuHandlesMap, handle);
					hParent = getParentHMENU(handle);
						
					ZeroMemory(&mii,sizeof(mii));
					mii.cbSize     = sizeof(mii);
					mii.fMask      = MIIM_STRING;
					mii.fType      = MFT_STRING;
					mii.dwTypeData = NULL;
					mii.cch        = 0;
					GetMenuItemInfo(hParent, pos, TRUE, &mii);
					mii.cch++;
					mii.dwTypeData = malloc(mii.cch);
						
					if (mii.dwTypeData)
					{
						GetMenuItemInfo(hParent, pos, TRUE, &mii);
						
						ZeroMemory(&rc,sizeof(rc));
						DrawText(hDC,mii.dwTypeData,mii.cch,&rc,DT_SINGLELINE|DT_VCENTER|DT_LEFT|DT_CALCRECT);
						
						lDims = GetMenuCheckMarkDimensions();
						wCheckWidth  = (WORD)(LOWORD(lDims)+Spacing);
						wCheckHeight = (WORD)(HIWORD(lDims)+Spacing);
						
						wCheckWidth  = (WORD)max(handle->bitmap->destsize.cx, wCheckWidth);
						wCheckHeight = (WORD)max(handle->bitmap->destsize.cy, wCheckHeight);
		
						lpMIS->itemWidth  = (rc.right-rc.left) + wCheckWidth     + (Spacing*3);	// Text width
						lpMIS->itemHeight = max((rc.bottom-rc.top),wCheckHeight) + (Spacing*2); // Text Height						
					}

					free(mii.dwTypeData);

					// Clean up resources
					SelectObject(hDC,hOldFont);
					DeleteObject(hFont);
					ReleaseDC(hWnd,hDC);
				} 
            }
        }
        break;
	case WM_DRAWITEM:
		{
			LPDRAWITEMSTRUCT lpDIS = (LPDRAWITEMSTRUCT) lParam;

			if (lpDIS->CtlType == ODT_MENU)
			{
				handle = getMenuHandle(pData->pMenuHandlesMap, lpDIS->itemID);
				
				if (handle && handle->bitmap)
				{
					long lDims;
					RECT rcFrame, rcBox, rcFill, rc;
					POINT pt;
					WORD wCheckWidth, wCheckHeight;
					WORD wWidth, wHeight;
					int nIndexDC;
					HBRUSH hFillBrush;
					struct CanvasHandle canvas;
					
					canvas.hDC = lpDIS->hDC;
					
					// Check box dimensions
					lDims = GetMenuCheckMarkDimensions();
					wCheckWidth  = LOWORD(lDims);
					wCheckHeight = HIWORD(lDims);
					 
					wWidth  = max(handle->bitmap->destsize.cx, wCheckWidth)  + (Spacing*2);
					wHeight = max(handle->bitmap->destsize.cy, wCheckHeight) + (Spacing*2);
					
					rcFrame.left  = lpDIS->rcItem.left;
					rcFrame.top   = lpDIS->rcItem.top;
					rcFrame.right = lpDIS->rcItem.bottom - lpDIS->rcItem.top;
					rcFrame.bottom= lpDIS->rcItem.top+wHeight;
					
					rcBox.left  = rcFrame.left+1;
					rcBox.top   = rcFrame.top+1;
					rcBox.right = rcFrame.right-1;
					rcBox.bottom= rcFrame.bottom-1;
					
					// Save off context attributes
					nIndexDC = SaveDC(lpDIS->hDC);
					
					// create brush for selection state
					if (lpDIS->itemState & ODS_SELECTED)
					{
						hFillBrush = GetSysColorBrush(COLOR_HIGHLIGHT);
						SetTextColor(lpDIS->hDC, GetSysColor(COLOR_HIGHLIGHTTEXT));
					}
					else
						hFillBrush = CreateSolidBrush(GetBkColor(lpDIS->hDC));
					
					SetBkMode(lpDIS->hDC,TRANSPARENT);
					
					// Add some spacing before drawing the text and hi-lite
					rcFill.left  = lpDIS->rcItem.left;
					rcFill.top   = lpDIS->rcItem.top-1;
					rcFill.right = lpDIS->rcItem.right;
					rcFill.bottom= lpDIS->rcItem.bottom;
					
					if ((lpDIS->itemState & ODS_SELECTED) && (lpDIS->itemState & ODS_CHECKED))
						rcFill.left+=wWidth+Spacing;
					
					FillRect(lpDIS->hDC, &rcFill, hFillBrush);
						
					rc = lpDIS->rcItem;
					rc.left += wWidth+Spacing;
					rc.right-= Spacing;
					
					// Draw out the bitmap associated with the menu item.
					pt.x = rcFrame.left+(((rcFrame.right-rcFrame.left) - handle->bitmap->destsize.cx)/2);
					pt.y = rcFrame.top +(((rcFrame.bottom-rcFrame.top) - handle->bitmap->destsize.cy)/2);

					osDrawBitmap(pt.x, pt.y, handle->bitmap, &canvas);
								
					// Draw bounding frame			
					if (lpDIS->itemState & ODS_SELECTED)
							DrawEdge(lpDIS->hDC,&rcFrame,BDR_RAISEDINNER,BF_RECT);
					
					rc.left+=Spacing;
					
					// Draw out menu item caption - text.					
					pos = getMenuPos(pData->pMenuHandlesMap, handle);
					hParent = getParentHMENU(handle);
						
					ZeroMemory(&mii,sizeof(mii));
					mii.cbSize     = sizeof(mii);
					mii.fMask      = MIIM_STRING;
					mii.fType      = MFT_STRING;
					mii.dwTypeData = NULL;
					mii.cch        = 0;
					GetMenuItemInfo(hParent, pos, TRUE, &mii);
					mii.cch++;
					mii.dwTypeData = malloc(mii.cch);
						
					if (mii.dwTypeData)
					{
						GetMenuItemInfo(hParent, pos, TRUE, &mii);
						
						if (lpDIS->itemState & ODS_GRAYED)
						{						
							// This will give it a disable text look 
							if (!(lpDIS->itemState & ODS_SELECTED))
							{
								SetTextColor(lpDIS->hDC, GetSysColor(COLOR_3DHILIGHT));			
								OffsetRect(&rc,1,1);
								DrawMenuText(lpDIS->hDC,mii.dwTypeData,&rc);
								OffsetRect(&rc,-1,-1);

								SetTextColor(lpDIS->hDC,GetSysColor(COLOR_GRAYTEXT));
								DrawMenuText(lpDIS->hDC,mii.dwTypeData,&rc);
							}
							else
								{
									COLORREF crGray = (GetSysColor(COLOR_GRAYTEXT) + RGB(64,64,64));
									SetTextColor(lpDIS->hDC,crGray);
									DrawMenuText(lpDIS->hDC,mii.dwTypeData,&rc);
								}
						}
						else
							DrawMenuText(lpDIS->hDC,mii.dwTypeData,&rc);
					}
						
					free(mii.dwTypeData);
					
					RestoreDC(lpDIS->hDC,nIndexDC);
				}
			}
		}
		return TRUE;
	case WM_INITMENUPOPUP:
	    {
	    	HMENU hMenu;
			int i, nCount;
			
			hMenu  = (HMENU) wParam;
			nCount = GetMenuItemCount((HMENU) wParam);
			for (i = 0; i < nCount; i++)
			{
				mii.cbSize = sizeof(mii);
				mii.fMask = MIIM_ID;
				GetMenuItemInfo(hMenu, i, TRUE, &mii);
				
				handle = getMenuHandle(pData->pMenuHandlesMap, (UINT) mii.wID);
				if (handle) handleMenuUpdate(handle);
			}
		}
	    break;
	}

	if (DocumentInterface == 1)
		return DefWindowProc (hWnd, uMsg, wParam, lParam);
	else
		return DefFrameProc (hWnd, pData ? pData->hClientWnd : NULL, uMsg, wParam, lParam);
};

LRESULT CALLBACK HMDIFrameFunction(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT result = HFrameSharedFunction(2, hWnd, uMsg, wParam, lParam);
	FrameData *pData = (FrameData *) GetWindowLong(hWnd,GWL_USERDATA);

	switch (uMsg)
	{
	case WM_CREATE:
		{
			HMENU hMenuBar, hMenuWindow;
			CLIENTCREATESTRUCT clientcreate;

			hMenuBar = CreateMenu ();				// Create the menu bar
			SetMenu (hWnd,hMenuBar);				// and associate it with the frame window
			hMenuWindow = CreatePopupMenu();		// Create the "Window" menu
			InsertMenu (hMenuBar,					// add it to the menuBar
						0xFFFFFFFF,					// at the end
						MF_BYPOSITION | MF_POPUP,	// Flags
						(UINT) hMenuWindow,			// the "Window" menu
						"&Window"					// and set its title
				);
			InsertMenu (hMenuWindow,0,MF_BYPOSITION | MF_STRING,OSMenuIDEnd+1,"Arrange &Icons");		// Add "Arrange Icons" command
			InsertMenu (hMenuWindow,0,MF_BYPOSITION | MF_STRING,OSMenuIDEnd+2,"&Tile Vertically");	// Add "Tile Vertically" command
			InsertMenu (hMenuWindow,0,MF_BYPOSITION | MF_STRING,OSMenuIDEnd+3,"Tile &Horizontally");	// Add "Tile Horizontally" command
			InsertMenu (hMenuWindow,0,MF_BYPOSITION | MF_STRING,OSMenuIDEnd+4,"&Cascade");			// Add "Cascade" command

			clientcreate.hWindowMenu  = hMenuWindow;
			clientcreate.idFirstChild = OSMenuIDEnd+5;	// Window ids must be generated from OSMenuIDEnd+5

			pData->hClientWnd = CreateWindow ("MDICLIENT",							// The MDICLIENT window class
										NULL,										// The window name
										MDIS_ALLCHILDSTYLES | WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE,
										0,0,										// position (x,y)
										0,0,										// size (w,h)
										hWnd,										// The frame window is the parent
										NULL,										// The menu (none at the moment)
										(HANDLE) ghModule,							// Instance that owns the window
										(LPVOID) &clientcreate						// The CLIENTCREATESTRUCT
										);
			if (!pData->hClientWnd)
				return -1;
				
			DrawMenuBar(hWnd);
		}
		break;
	case WM_COMMAND:
		if (HIWORD (wParam)==0 && lParam!=0)
		{
		}
		else
			switch (wParam)
			{
				case (OSMenuIDEnd+1):
					SendMessage (pData->hClientWnd,WM_MDIICONARRANGE,0,0);
					break;
				case (OSMenuIDEnd+2):
					SendMessage (pData->hClientWnd,WM_MDITILE,(WPARAM) MDITILE_VERTICAL,0);
					break;
				case (OSMenuIDEnd+3):
					SendMessage (pData->hClientWnd,WM_MDITILE,(WPARAM) MDITILE_HORIZONTAL,0);
					break;
				case (OSMenuIDEnd+4):
					SendMessage (pData->hClientWnd,WM_MDICASCADE,0,0);
					break;
			}
		break;
	}

	return result;
};

LRESULT CALLBACK HSDIFrameFunction(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	LRESULT result = HFrameSharedFunction(1, hWnd, uMsg, wParam, lParam);
	FrameData *pFrameData = (FrameData *) GetWindowLong(hWnd,GWL_USERDATA);

	switch (uMsg)
	{
	case WM_CREATE:
		SetMenu(hWnd, CreateMenu());
		break;
	case WM_ACTIVATE:
		if (pFrameData->hClientWnd)
		{
			if (wParam == WA_INACTIVE)
			{
				if (gInKey)
					handleWindowKeyboard(pFrameData->hClientWnd, evKeyLost, gCurChar, GetModifiers());
				gInKey = FALSE;
				gCurChar = 0;
				handleWindowDeactivate(pFrameData->hClientWnd);
			}
			else
			{
				handleWindowActivate(pFrameData->hClientWnd);
			}
		}
        break;
	}

	return result;
};
