#include "ListBox.h"
#include "Internals.h"
#include "Handlers_stub.h"

WindowHandle osCreateListBox(WindowHandle window, BOOL multisel)
{
	HWND hListBox;

	hListBox = CreateWindowEx(
			  WS_EX_CLIENTEDGE,
			  "LISTBOX",
			  NULL,
			  WS_CHILD | WS_VSCROLL | WS_BORDER | LBS_NOTIFY | WS_TABSTOP | (multisel ? LBS_EXTENDEDSEL : 0),
			  0,0,0,0,
			  window,
			  NULL,
			  ghModule,
			  NULL
			);
	return checkWindow(hListBox, "LISTBOX");
};

void osAppendListBoxItem(WindowHandle listbox, char *title)
{
	SendMessage(listbox, LB_ADDSTRING, 0, (LPARAM) title);
	osForceContainerReLayout(listbox);
};

void osInsertListBoxItem(WindowHandle listbox, int index, char *title)
{
	SendMessage(listbox, LB_INSERTSTRING, index, (LPARAM) title);
	osForceContainerReLayout(listbox);
};

void osRemoveListBoxItem(WindowHandle listbox, int index)
{
	SendMessage(listbox, LB_DELETESTRING, index, 0);
	osForceContainerReLayout(listbox);
};

void osRemoveAllListBoxItems(WindowHandle listbox)
{
	SendMessage(listbox, LB_RESETCONTENT,0,0);
	osForceContainerReLayout(listbox);
};

void osGetListBoxReqSize(WindowHandle listbox, int *res)
{
	SIZE sz;
	int i,nCount, nLen, width, height;
	char *buffer;
	HDC hDC = GetDC(listbox);
	HFONT hFont = (HFONT) SendMessage(listbox,WM_GETFONT,0,0);

	if (hFont)
		SelectObject(hDC, hFont);

	width  = 10;
	height = 2;
	nCount = SendMessage(listbox,LB_GETCOUNT,0,0);
	for (i = 0; i < nCount; i++)
	{
		nLen = SendMessage(listbox,LB_GETTEXTLEN,i,0);
		buffer = (char *) rmalloc(nLen+1);
		nLen = SendMessage(listbox,LB_GETTEXT,i,(LPARAM)buffer);

		GetTextExtentPoint32(hDC, buffer, nLen, &sz);

		rfree(buffer);

		if (width < sz.cx) width = sz.cx;
		height += sz.cy;
	}

	ReleaseDC(listbox, hDC);

	res[0] = width + GetSystemMetrics(SM_CXBORDER)*2 + GetSystemMetrics(SM_CXVSCROLL);
	res[1] = height+ GetSystemMetrics(SM_CYBORDER)*2;
};

int osGetListBoxSingleSelection(WindowHandle listbox)
{
	return SendMessage(listbox, LB_GETCURSEL, 0, 0);
};

BOOL osGetListBoxItemSelectState(WindowHandle listbox, int index)
{
	return SendMessage(listbox, LB_GETSEL, index, 0) > 0;
}

void osSetListBoxSingleSelection(WindowHandle listbox, int index)
{
	SendMessage(listbox, LB_SETCURSEL, index, 0);
	handleControlCommand(listbox);
};

void osSetListBoxItemSelectState(WindowHandle listbox, int index, BOOL state)
{
	SendMessage(listbox, LB_SETSEL, state, index);
};

int osGetListBoxCurrentItem(WindowHandle listbox)
{
	return SendMessage(listbox, LB_GETCARETINDEX, 0, 0);
}
