#include "Menu.h"
#include "Window.h"
#include "Internals.h"
#include "Handlers_stub.h"


#define CHECK_MENU_TYPE(handle,mask,ret) \
   if (((handle ? handle->type : MENU_SUBMENU) & (mask)) == 0) \
   { \
   		printf("Invalid menu handle type."); \
   		return ret; \
   }
   
#define CHECK_MENU_TYPE_V(handle,mask) \
   if (((handle ? handle->type : MENU_SUBMENU) & (mask)) == 0) \
   { \
   		printf("Invalid menu handle type."); \
   		return; \
   }

static void AddAccelString(int nKeyCode, int mods, char *text)
{
	if (!nKeyCode)
		return;

	strcat(text, "\t");

	if (mods & altBIT  ) strcat(text, "Alt+");
	if (mods & ctrlBIT ) strcat(text, "Ctrl+");
	if (mods & shiftBIT) strcat(text, "Shift+");

	switch (nKeyCode)
	{
	case kbBackSpace: 	strcat(text, "BackSpace");	break;
	case kbTab: 		strcat(text, "Tab");		break;
	case kbEnter: 		strcat(text, "Enter");		break;
	case kbEscape:		strcat(text, "Esc");		break;
	case kbBegin: 		strcat(text, "Home");		break;
	case kbClear: 		strcat(text, "Clear");		break;
	case kbDelete:		strcat(text, "Del");		break;
	case kbDown: 		strcat(text, "Down");		break;
	case kbEnd: 		strcat(text, "End");		break;
	case kbF1: 			strcat(text, "F1");			break;
	case kbF2: 			strcat(text, "F2");			break;
	case kbF3: 			strcat(text, "F3");			break;
	case kbF4: 			strcat(text, "F4");			break;
	case kbF5: 			strcat(text, "F5");			break;
	case kbF6: 			strcat(text, "F6");			break;
	case kbF7: 			strcat(text, "F7");			break;
	case kbF8: 			strcat(text, "F8");			break;
	case kbF9: 			strcat(text, "F9");			break;
	case kbF10:			strcat(text, "F10");		break;
	case kbF11:			strcat(text, "F11");		break;
	case kbF12:			strcat(text, "F12");		break;
	case kbF13:			strcat(text, "F13");		break;
	case kbF14:			strcat(text, "F14");		break;
	case kbF15:			strcat(text, "F15");		break;
	case kbHelp: 		strcat(text, "Help");		break;
	case kbLeft: 		strcat(text, "Left");		break;
	case kbPgDown: 		strcat(text, "PgDown");		break;
	case kbPgUp: 		strcat(text, "PgUp");		break;
	case kbRight: 		strcat(text, "Right");		break;
	case kbUp: 			strcat(text, "Up");			break;
	case ' ':			strcat(text, "Space");		break;
	default:
		{
			char s[2];

			if (nKeyCode >= 256)
			{
				strcat(text,"Alt+");
				nKeyCode -= 256;
			}

			if (iscntrl(nKeyCode))
			{
				nKeyCode += '@';
				strcat(text,"Ctrl+");
			}
			else
				if (isupper(nKeyCode)) strcat(text,"Shift+");
				else nKeyCode = toupper(nKeyCode);

			s[0] = (char) nKeyCode;
			s[1] = 0;
			strcat(text, s);
		}
		break;
	}
}

static void updateMenuBar(MenuHandle parent)
{
	if (!parent)
	{
		RECT rc;
		FrameData *pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
		
		GetClientRect(ghWndFrame, &rc);
		SetWindowPos(pFrameData->hClientWnd,NULL,rc.left,rc.top,rc.right-rc.left,rc.bottom-rc.top,SWP_NOZORDER);
		DrawMenuBar(ghWndFrame);
	}
}

MenuHandle osCreatePopupMenu()
{	
	MenuHandle handle;
	FrameData *pFrameData;
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, NULL, MENU_POPUP, -1);

	if (handle)
		handle->hMenu = CreatePopupMenu();

	return handle;
};

void osTrackPopupMenu(MenuHandle menu, WindowHandle hWnd, int x, int y)
{
	POINT pos;
	int i, nCount;
	MenuHandle handle;
	MENUITEMINFO mii;
	FrameData *pData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);

	pos.x = x;
	pos.y = y;
	ClientToScreen(hWnd, &pos);

	nCount = GetMenuItemCount(menu->hMenu);
	for (i = 0; i < nCount; i++)
	{
		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_ID;
		GetMenuItemInfo(menu->hMenu, i, TRUE, &mii);

		handle = getMenuHandle(pData->pMenuHandlesMap, (UINT) mii.wID);
		if (handle) handleMenuUpdate(handle);
	}

	TrackPopupMenu(menu->hMenu, TPM_LEFTALIGN | TPM_TOPALIGN, pos.x, pos.y, 0, ghWndFrame, NULL);
}

MenuHandle osInsertMenu(MenuHandle parent, int pos)
{	
	MenuHandle handle;
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(parent, MENU_SUBMENU | MENU_POPUP, NULL);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, parent, MENU_SUBMENU, pos);
		
	if (handle)
	{
		handle->hMenu = CreateMenu();

		InsertMenu(getParentHMENU(handle),getMenuPos(pFrameData->pMenuHandlesMap, handle),MF_BYPOSITION | MF_POPUP,(UINT)handle->hMenu,"");

		updateMenuBar(parent);
	}

	return handle;
};

MenuHandle osInsertMenuItem(MenuHandle parent, int pos)
{
	MenuHandle handle;
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(parent, MENU_SUBMENU | MENU_POPUP, NULL);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, parent, MENU_ITEM, pos);
	
	if (handle)
	{
		InsertMenu(getParentHMENU(handle),getMenuPos(pFrameData->pMenuHandlesMap, handle),MF_BYPOSITION | MF_STRING,handle->id,"");
		updateMenuBar(parent);
	}
	
	return handle;
};

MenuHandle osInsertMenuCheckItem(MenuHandle parent, int pos)
{
	MenuHandle handle;
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(parent, MENU_SUBMENU | MENU_POPUP, NULL);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, parent, MENU_CHECK_ITEM, pos);
	
	if (handle)
	{
		InsertMenu(getParentHMENU(handle),getMenuPos(pFrameData->pMenuHandlesMap, handle),MF_BYPOSITION | MF_STRING,handle->id,"");
		updateMenuBar(parent);
	}
	
	return handle;
};

MenuHandle osInsertMenuSeparatorItem(MenuHandle parent, int pos)
{
	MenuHandle handle;
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(parent, MENU_SUBMENU | MENU_POPUP, NULL);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, parent, MENU_SEPARATOR, pos);
	
	if (handle)
	{
		InsertMenu(getParentHMENU(handle),getMenuPos(pFrameData->pMenuHandlesMap, handle),MF_BYPOSITION | MF_SEPARATOR,0,NULL);
		updateMenuBar(parent);
	}
	
	return handle;
}

MenuHandle osInsertMenuRadioItem(MenuHandle parent, int pos)
{
	MenuHandle handle;
	FrameData *pFrameData;
	MENUITEMINFO mii;
	
	CHECK_MENU_TYPE(parent, MENU_SUBMENU | MENU_POPUP, NULL);
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle = newMenuHandle(pFrameData->pMenuHandlesMap, parent, MENU_RADIO_ITEM, pos);
	if (handle)
	{
		mii.cbSize = sizeof(mii);
		mii.fMask  = MIIM_ID | MIIM_STATE | MIIM_TYPE;
		mii.wID    = handle->id;
		mii.fState = MFS_ENABLED;
		mii.fType  = MFT_RADIOCHECK | MFT_STRING;
		mii.dwTypeData = (DWORD) "";
		mii.cch    = 0;
		InsertMenuItem(getParentHMENU(handle),getMenuPos(pFrameData->pMenuHandlesMap, handle),TRUE,&mii);
		updateMenuBar(parent);
	}
	return handle;
};

void osSetMenuRadioGroup(MenuHandle *handles)
{
	MenuHandle child, handle, *phandle;
	
	if (!(handles && *handles))
		return;

	phandle=handles;
	for (;;)
	{
		handle = *phandle;
		
		child = handle->nextInGroup;
		while (child->nextInGroup != handle)
			child = child->nextInGroup;
		child->nextInGroup = handle->nextInGroup;
		
		phandle++;
		if (*phandle)
			handle->nextInGroup = *phandle;
		else
		{
			handle->nextInGroup = *handles;
			break;
		}
	}
}

void osDestroyMenu(MenuHandle handle)
{
	FrameData *pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);

	notifyHandleForDestroy(handle);

	if (handle->type != MENU_POPUP)
	{
		DeleteMenu(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), MF_BYPOSITION);
		updateMenuBar(handle->parent);
	}

	deleteMenuHandle(pFrameData->pMenuHandlesMap, handle);
}

int osGetMenuItemCount(MenuHandle handle)
{
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(handle, MENU_SUBMENU | MENU_POPUP, 0);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	
	return getChildrenCount(pFrameData->pMenuHandlesMap, handle);
}

void osSetMenuItemEnabled(MenuHandle handle, BOOL bState)
{
	if (handle->type != MENU_POPUP)
	{
		FrameData *pFrameData;

		CHECK_MENU_TYPE_V(handle, MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM | MENU_SUBMENU);

		pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);

		EnableMenuItem(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), (bState ? MF_ENABLED : MF_GRAYED) | MF_BYPOSITION);
		updateMenuBar(handle->parent);
	}
};

BOOL osGetMenuItemEnabled(MenuHandle handle)
{
	if (handle->type != MENU_POPUP)
	{
		MENUITEMINFO mii;
		FrameData *pFrameData;

		CHECK_MENU_TYPE(handle, MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM | MENU_SUBMENU, FALSE);

		pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);	

		mii.cbSize = sizeof(mii);
		mii.fMask = MIIM_STATE;
		GetMenuItemInfo(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), TRUE, &mii);
		return (mii.fState & MFS_DISABLED) == 0;
	}
	else
		return TRUE;
};

void osSetMenuItemChecked(MenuHandle handle, BOOL bState)
{
	FrameData *pFrameData;
	MenuHandle child;
	
	CHECK_MENU_TYPE_V(handle, MENU_CHECK_ITEM | MENU_RADIO_ITEM);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	
	if (bState)
	{
		child = handle->nextInGroup;
		while (child != handle)
		{
			CheckMenuItem(getParentHMENU(child), getMenuPos(pFrameData->pMenuHandlesMap, child), MF_UNCHECKED | MF_BYPOSITION);
			child = child->nextInGroup;
		}
		
		CheckMenuItem(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), MF_CHECKED   | MF_BYPOSITION);
	}
	else
		CheckMenuItem(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), MF_UNCHECKED | MF_BYPOSITION);

	updateMenuBar(handle->parent);
	
	handleMenuCommand(handle);
};

BOOL osGetMenuItemChecked(MenuHandle handle)
{
	MENUITEMINFO mii;
	FrameData *pFrameData;
	
	CHECK_MENU_TYPE(handle, MENU_CHECK_ITEM | MENU_RADIO_ITEM, FALSE);
	
	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	
	mii.cbSize = sizeof(mii);
	mii.fMask = MIIM_STATE;
	GetMenuItemInfo(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), TRUE, &mii);
	return (mii.fState & MFS_CHECKED) != 0;
}

char *osGetMenuLabel(MenuHandle handle)
{
	if (handle->type != MENU_POPUP)
	{
		int pos;
		HMENU hParent;
		char *s;
		MENUITEMINFO mii;
		FrameData *pFrameData;

		CHECK_MENU_TYPE(handle, MENU_SUBMENU | MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM, NULL);

		pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
		pos = getMenuPos(pFrameData->pMenuHandlesMap, handle);
		hParent = getParentHMENU(handle);

		memset(&mii,0,sizeof(mii));
		mii.cbSize     = sizeof(mii);
		mii.fMask      = MIIM_STRING;
		mii.fType      = MFT_STRING;
		mii.dwTypeData = NULL;
		mii.cch        = 0;
		GetMenuItemInfo(hParent, pos, TRUE, &mii);
		mii.cch++;
		mii.dwTypeData = malloc(mii.cch);

		if (mii.dwTypeData)
		{
			GetMenuItemInfo(hParent, pos, TRUE, &mii);

			s = mii.dwTypeData;
			while (*s && *s != '\t') s++;
			*s = 0;
		}

		return mii.dwTypeData;
	}
	else
		return NULL;
}

void osSetMenuLabel(MenuHandle handle, char *title)
{
	if (handle->type != MENU_POPUP)
	{
		char *s, *temp;
		MENUITEMINFO mii;
		FrameData *pFrameData;

		CHECK_MENU_TYPE_V(handle, MENU_SUBMENU | MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM);

		pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);

		if (handle->parent == NULL)
			temp = title;
		else
		{
			temp = malloc(strlen(title)+32);
			if (!temp)
				temp = title;
			else
			{
				s = temp;
				while (*title)
				{
					if (*title != '\t') *s++ = *title;
					title++;
				}
				*s = 0;
				AddAccelString(handle->key, handle->mods, temp);
			}
		}

		memset(&mii,0,sizeof(mii));
		mii.cbSize     = sizeof(mii);
		mii.fMask      = MIIM_STRING;
		mii.fType      = MFT_STRING;
		mii.dwTypeData = temp;
		mii.cch        = strlen(temp);
		SetMenuItemInfo(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), TRUE, &mii);

		updateMenuBar(handle->parent);

		if (temp != title) free(temp);
	}
}

void osSetMenuItemAccel(MenuHandle handle, int key, unsigned int mods)
{
	int pos;
	HMENU hParent;
	char *s;
	MENUITEMINFO mii;
	FrameData *pFrameData;

	CHECK_MENU_TYPE_V(handle, MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM);

	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	pos = getMenuPos(pFrameData->pMenuHandlesMap, handle);
	hParent = getParentHMENU(handle);

	updateAccelTable(pFrameData->pMenuHandlesMap, handle, key, mods);

	if (handle->parent != NULL)
	{
		memset(&mii,0,sizeof(mii));
		mii.cbSize     = sizeof(mii);
		mii.fMask      = MIIM_STRING;
		mii.fType      = MFT_STRING;
		mii.dwTypeData = NULL;
		mii.cch        = 0;
		GetMenuItemInfo(hParent, pos, TRUE, &mii);
		mii.dwTypeData = malloc(mii.cch+32);

		if (!mii.dwTypeData)
			return;

		mii.cch++;
		GetMenuItemInfo(hParent, pos, TRUE, &mii);

		s = mii.dwTypeData;
		while (*s && *s != '\t') s++;
		*s = 0;

		AddAccelString(key, mods, s);

		mii.cch = strlen(mii.dwTypeData);
		SetMenuItemInfo(hParent, pos, TRUE, &mii);

		updateMenuBar(handle->parent);

		free(mii.dwTypeData);
	}
}

void osGetMenuItemAccel(MenuHandle handle, int *key, unsigned int *mods)
{
	CHECK_MENU_TYPE_V(handle, MENU_RADIO_ITEM | MENU_CHECK_ITEM | MENU_ITEM);

	*key  = handle->key;
	*mods = handle->mods;
}

void osSetMenuItemBitmap(MenuHandle handle, BitmapHandle bitmap)
{
	MENUITEMINFO mii;
	FrameData *pFrameData;

	CHECK_MENU_TYPE_V(handle, MENU_ITEM);

	pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	handle->bitmap = bitmap;

	memset(&mii,0,sizeof(mii));
	mii.cbSize     = sizeof(mii);
	mii.fMask      = MIIM_FTYPE;
	mii.fType      = bitmap ? MFT_OWNERDRAW : MFT_STRING;
	SetMenuItemInfo(getParentHMENU(handle), getMenuPos(pFrameData->pMenuHandlesMap, handle), TRUE, &mii);

	updateMenuBar(handle->parent);
}

int osGetMenuItemPos(MenuHandle handle)
{
	FrameData *pFrameData = (FrameData *) GetWindowLong(ghWndFrame,GWL_USERDATA);
	return getMenuPos(pFrameData->pMenuHandlesMap, handle);
}
