#ifndef MENU_ITEMS_MAP_H
#define MENU_ITEMS_MAP_H

#include "Types.h"

#define HASH_BLOCK_SIZE 10
#define HASH_TABLE_SIZE 17

enum MENU_TYPE
	{ MENU_POPUP       = 1
	, MENU_SUBMENU     = 2
	, MENU_SEPARATOR   = 4
	, MENU_ITEM        = 8	
	, MENU_CHECK_ITEM  = 16
	, MENU_RADIO_ITEM  = 32
	};
	
typedef enum MENU_TYPE MENU_TYPE;

struct MenuHandle
{
	struct MenuHandle *next;
	struct MenuHandle *parent;
	struct MenuHandle *children;
	struct MenuHandle *sibling;
	struct MenuHandle *nextInGroup;

	UINT id;
	HMENU hMenu;
	int key, mods;
	BitmapHandle bitmap;
	MENU_TYPE type;
};

typedef struct Block_tag
{
	struct Block_tag *pNext;
} Block;

typedef struct
{
	struct MenuHandle *children;
	struct MenuHandle *popups;
	
	Block* pBlocks;
	MenuHandle pFreeList;
	MenuHandle HashTable[HASH_TABLE_SIZE];
	HACCEL hAccelTable;
	int nAccelCount;
	UINT nMenuID;
} MenuHandlesMap;

MenuHandlesMap *newMenuHandlesMap();
void deleteMenuHandlesMap(MenuHandlesMap *pMap);
MenuHandle newMenuHandle(MenuHandlesMap *pMap, MenuHandle parent, MENU_TYPE type, int pos);
void deleteMenuHandle(MenuHandlesMap *pMap, MenuHandle handle);
MenuHandle getMenuHandle(MenuHandlesMap *pMap, UINT id);
int getMenuPos(MenuHandlesMap *pMap, MenuHandle handle);
void updateAccelTable(MenuHandlesMap *pMap, MenuHandle item, int  key, unsigned int  mods);
HACCEL getAccelTableFromMap(MenuHandlesMap *pMap);
HMENU getParentHMENU(MenuHandle handle);
int getChildrenCount(MenuHandlesMap *pMap, MenuHandle handle);
MenuHandle getNthChild(MenuHandlesMap *pMap, MenuHandle handle, int index);
void notifyHandleForDestroy(MenuHandle handle);
void notifyAllHandlesForDestroy(MenuHandlesMap *pMap);

#endif
