#ifndef TYPES_H
#define TYPES_H

#include "config.h"
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <memory.h>
#include <string.h>

typedef enum {PosLeft=0, PosTop=1, PosRight=2, PosBottom=3} PositionType;

#ifdef WIN32_TARGET

#define _WIN32_IE 0x0501
#include <windows.h>
#include <mmsystem.h>
#include <shlobj.h>

#ifndef TB_GETBUTTONINFO
#define TB_GETBUTTONINFO (WM_USER + 65)
#endif

#ifndef I_IMAGENONE
#define I_IMAGENONE -2
#endif

typedef HWND WindowHandle;

typedef struct
{
	UINT_PTR id;
	int interval;
	BOOL enabled;
} *TimerHandle;

typedef struct
{
	HBITMAP hBitmap;
	SIZE sourcesize;
	SIZE destsize;
} *BitmapHandle;

typedef HFONT FontHandle;
typedef struct MenuHandle *MenuHandle;

typedef struct ToolHandle *ToolHandle;

typedef struct
{
	int count, max_count;
	POINT data[0];
} *PolygonHandle;

typedef struct CanvasHandle
{
	HDC hDC;
	HDC hBufferedDC;
	HBITMAP hBufferBitmap;
	BOOL bInvalidated;
	HPEN thePen;
	HBRUSH theBrush;
	HFONT theFont;

	DWORD stylesCount;
	DWORD *stylesPtr;
} *CanvasHandle;

typedef void *CodecsEnumeratorHandle;

#else

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <pango/pango.h>
#include <pango/pangox.h>
#include <gnome.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>

typedef GtkWidget   *WindowHandle;

typedef struct
{
	guint id;
	int interval;
	gboolean enabled;
} *TimerHandle;

typedef struct
{
	GdkPixbuf *pixbuf;
	int width;
	int height;
} *BitmapHandle;

typedef struct
{
	PangoFontDescription *font_descr;
	PangoFontMetrics *metrics;
	gint style;
} *FontHandle;

typedef GtkWidget *MenuHandle;
typedef GtkWidget *MenuItemHandle;

typedef GtkWidget *ToolHandle;

typedef struct
{
	int count, max_count;
	GdkPoint data[0];
} *PolygonHandle;

typedef struct
{
	GdkDrawable *drawable;
	GdkGC *theDrawGC, *theFillGC, *theEraseGC, *theTextGC;
	FontHandle theFont;
	gint lineCustomCount;
	gint8 *lineCustomDashes;
	GdkColor penColor;
	GdkColor backColor;
	gboolean backDraw;
	GdkRegion *region;
	gboolean buffered;
	GdkPixbuf *pixbuf;
	GdkPixmap *tile;
	PangoLayout *layout;
} *CanvasHandle;

typedef gboolean BOOL;

typedef struct
{
	BOOL hasCurrent;
	GSList *first;
	GSList *current;
} *CodecsEnumeratorHandle;

#endif

extern void osInit(char *appName, char *appVersion, int DocumentInterface);
extern void osStart();
extern void osQuit();

#endif
